SystemOrganization addCategory: #'SmallDEVS-SimSpace'!
SystemOrganization addCategory: #'SmallDEVS-Examples'!
SystemOrganization addCategory: #'SmallDEVS-Core'!
SystemOrganization addCategory: #'SmallDEVS-Continuous'!
SystemOrganization addCategory: #'SmallDEVS-RT'!
SystemOrganization addCategory: #'SmallDEVS-Prototypes'!

Delay subclass: #Timer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-RT'!

!Timer commentStamp: 'vj 8/13/2003 01:04' prior: 0!
Waiting for real time value. How to use:

	| t aTime |
	aTime _ Time millisecondClockValue + 10000.
	t _ Timer at: aTime.
	t wait.

	"waiting can be stopped by t unschedule"



!

!Timer methodsFor: 'as yet unclassified' stamp: 'vj 7/24/2003 19:16'!
unschedule

	super unschedule.
	delaySemaphore signal.
! !

!Timer methodsFor: 'as yet unclassified' stamp: 'vj 8/13/2003 01:06'!
scheduleResumptionTime: t

	"This code is taken from Delay and modified"

	beingWaitedOn ifTrue: [self error: 'This Delay has already been scheduled.'].
	AccessProtect critical: [
		beingWaitedOn _ true.
		resumptionTime _ t.           "as oposed to Time millisecondClockValue + delayDuration."
		ActiveDelay == nil
			ifTrue: [self activate]
			ifFalse: [
				resumptionTime < ActiveDelay resumptionTime
					ifTrue: [
						SuspendedDelays add: ActiveDelay.
						self activate]
					ifFalse: [SuspendedDelays add: self]]].
! !

!Timer class methodsFor: 'as yet unclassified' stamp: 'vj 7/24/2003 17:56'!
at: time

	^ (self new setDelay: 0 forSemaphore: Semaphore new) scheduleResumptionTime: time! !

!Timer methodsFor: 'as yet unclassified' stamp: 'vj 7/24/2003 18:09'!
wait

	delaySemaphore wait.
! !

Object subclass: #AbstractSimulation
	instanceVariableNames: 'parent name process idleSemaphore startStopSemaphore accessProtect clock stopTime reportStream verbose'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-SimSpace'!

!AbstractSimulation methodsFor: 'as yet unclassified' stamp: 'vj 11/20/2004 19:17'!
parent
	^ parent! !

!AbstractSimulation methodsFor: 'simulation control' stamp: 'vj 11/19/2004 01:49'!
start

	| r |
	accessProtect critical: [ r _ self startUnprotected ].
	^ r
! !

AbstractSimulation subclass: #DEVSRootSolver
	instanceVariableNames: 'rootDEVS'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Core'!

!DEVSRootSolver methodsFor: 'simulation' stamp: 'vj 11/25/2004 15:07'!
simulationStep

	clock _ rootDEVS timeNext.
	reportStream nextPutAll: '****************** time: ', clock printString, String cr.
	rootDEVS receiveMsgStarTime: clock. 				"send (*, t) to root DEVS"
	verbose ifTrue: [
		reportStream nextPutAll: '* Root DEVS Output Port Configuration: ', String cr.
	 	rootDEVS reportOutputPortConfig ].
	reportStream flush.


! !

!DEVSRootSolver methodsFor: 'as yet unclassified' stamp: 'vj 11/21/2004 16:38'!
beVerbose: truefalse

	verbose _ truefalse.
	rootDEVS beVerbose: truefalse.

! !

!DEVSRootSolver methodsFor: 'as yet unclassified' stamp: 'vj 8/1/2003 22:04'!
rootDEVS: d

	rootDEVS _ d.
	rootDEVS parent: self.


! !

DEVSRootSolver subclass: #DEVnDESSRootSolver
	instanceVariableNames: 'tn h dh seDetected'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Continuous'!

!DEVnDESSRootSolver methodsFor: 'simulation' stamp: 'vj 8/5/2003 19:30'!
receiveMsgSETime: tnc

	seDetected _ true.
	tnc < tn ifTrue: [ tn _ tnc ] ifFalse: [ (tnc closeTo: tn) ifFalse: [ self error: 'Bad sync !!'] ]

! !

!DEVnDESSRootSolver methodsFor: 'OBSOLETE ???' stamp: 'vj 11/19/2004 20:59'!
simulateB: t

	stopTime _ t.

	[ clock < stopTime ] whileTrue: [ 
		self bigStep ].

	rootDEVS reportStream flush.

! !

!DEVnDESSRootSolver methodsFor: 'OBSOLETE ???' stamp: 'vj 11/19/2004 20:23'!
bigStep

		tn _ rootDEVS timeNext. 

		[ (clock < tn) & (clock < stopTime) ] whileTrue: [ 
			self numericStep ].
		rootDEVS reportStream nextPutAll: String cr.

		tn <= stopTime ifTrue: [ 
			clock _ tn.
			rootDEVS reportStream nextPutAll: 
				'****************** time: ', clock printString, String cr.
			rootDEVS receiveMsgStarTime: clock. 				"send (*, t) to root DEVS"
			rootDEVS verbose ifTrue: [
				rootDEVS reportStream nextPutAll: 
					'* Root DEVS Output Port Configuration: ', String cr.
		 		rootDEVS reportOutputPortConfig ]].

! !

!DEVnDESSRootSolver methodsFor: 'simulation' stamp: 'vj 11/20/2004 20:39'!
initialize

	super initialize.
	h _ 0.01.
! !

!DEVnDESSRootSolver methodsFor: 'simulation' stamp: 'vj 11/25/2004 15:21'!
simulationStep

	"Numeric step:"
	tn _ rootDEVS timeNext. 
	(clock < tn) & (clock < stopTime) ifTrue: [ ^ self numericStep ].

	"Discrete step:"
	reportStream nextPutAll: String cr.
	tn <= stopTime ifTrue: [ 
		clock _ tn.
		reportStream nextPutAll: '****************** time: ', clock printString, String cr.
		rootDEVS receiveMsgStarTime: clock. 				"send (*, t) to root DEVS"
		verbose ifTrue: [
			reportStream nextPutAll: '* Root DEVS Output Port Configuration: ', String cr.
	 		rootDEVS reportOutputPortConfig ]].

! !

!DEVnDESSRootSolver methodsFor: 'simulation' stamp: 'vj 11/25/2004 15:22'!
numericStep

	clock + h < tn ifTrue: [ dh _ h ] ifFalse: [ dh _ tn - clock ].
	seDetected _ false.
	rootDEVS receiveNMsgStarTime: clock step: dh.		"send (*, t, h) to root DEVS"
	rootDEVS receiveNMsgX: {} time: clock step: dh.		"send (x, t, h) to root DEVS"
	reportStream nextPutAll: '.'.
	seDetected ifTrue: [ 	| dhmin dhmax | dhmin _ 0. dhmax _ dh.
		[ seDetected & ((dhmax - dhmin) < 0.001) ] whileFalse: [ 
			reportStream nextPutAll: ':'.
			seDetected ifTrue: [ dhmax _ dh ] ifFalse: [ dhmin _ dh ].
			dh _ (dhmax + dhmin)/2.
			seDetected _ false.
			rootDEVS receiveNMsgStarTime: clock step: dh.		
			rootDEVS receiveNMsgX: {} time: clock step: dh.
				"Note: rootDEVS and consequently an AtomicDEVnDESS is repeatedly called 
				 for the same time. It is supposed that it remembers state and time 
				 from which last step was made. It must check time for which it is called
				 and either revert to previous state or save current state before making a
				 step." ] ].
	clock _ clock + dh
! !

!DEVSRootSolver methodsFor: 'simulation' stamp: 'vj 11/20/2004 16:39'!
prepareToStop

	rootDEVS receiveMsgSync.
! !

!DEVSRootSolver methodsFor: 'as yet unclassified' stamp: 'vj 11/20/2004 20:39'!
initialize

	super initialize.
	rootDEVS receiveMsgInitTime: clock.
! !

!DEVSRootSolver methodsFor: 'as yet unclassified' stamp: 'vj 11/20/2004 19:25'!
finalize

	"who needs this?????"
	self halt. "super finalize ???"
	rootDEVS finalize! !

!DEVSRootSolver methodsFor: 'simulation' stamp: 'vj 11/17/2004 23:52'!
time

	^ clock
! !

!DEVSRootSolver methodsFor: 'as yet unclassified' stamp: 'vj 8/23/2003 19:34'!
rootSolver

	^ self! !

!DEVSRootSolver methodsFor: 'simulation' stamp: 'vj 11/20/2004 16:38'!
prepareToStart

	rootDEVS receiveMsgInitTime: clock.
! !

!DEVSRootSolver methodsFor: 'as yet unclassified' stamp: 'vj 8/26/2003 00:09'!
rootDEVS

	^ rootDEVS


! !

!DEVSRootSolver methodsFor: 'as yet unclassified' stamp: 'vj 11/21/2004 16:38'!
deepVerbose: truefalse

	verbose _ truefalse.
	rootDEVS deepVerbose: truefalse.

! !

DEVSRootSolver subclass: #DEVSRootSolverRT
	instanceVariableNames: 'rtTimer startTime inputQueue seQueue rtFactor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-RT'!

!DEVSRootSolverRT methodsFor: 'realtime simulation' stamp: 'vj 11/20/2004 19:27'!
initialize

	super initialize.	
	inputQueue _ SharedQueue new.
	seQueue _ SharedQueue new.	
	startTime _ Time millisecondClockValue.
	rtFactor _ 1.

! !

!DEVSRootSolverRT methodsFor: 'realtime simulation' stamp: 'vj 11/25/2004 15:08'!
simulationStep

	"TO DO: make it more consistent with DEVnDESS solver"

	| msg aSubModel duration inputOccured input |

	"Synchronize with real time"
	rtTimer _ Timer at: ((startTime + ((rootDEVS timeNext min: stopTime) / rtFactor * 1000)) min: SmallInteger maxVal) rounded.
	rtTimer wait.		"Waiting can be interrupted by asynchronous internal or external event."

	"Step"
	clock _ self realTime min: rootDEVS timeNext. 
	reportStream nextPutAll: '****************** simulation time, real time: ', 
		clock asFloat printString, ', ', self realTime asFloat printString, String cr.
	duration _ Time millisecondsToRun: [
		inputOccured _ false.
		input _ Dictionary new.
		[ (msg _ inputQueue nextOrNil) == nil ] whileFalse: [ 
			input at: (rootDEVS inputPortNamed: msg first) put: msg second.
			inputOccured _ true ].
		[ (aSubModel _ seQueue nextOrNil) == nil ] whileFalse: [ 
			aSubModel sendMsgSETime: clock ].
		inputOccured
		   ifTrue: [
			rootDEVS receiveMsgX: input time: clock.	 "... put input & update event lists"
			verbose ifTrue: [
				reportStream nextPutAll: '* Root DEVS Input Port Configuration: ', String cr.			
				rootDEVS reportInputPortConfig ]]
		   ifFalse: [
			rootDEVS timeNext <= clock ifTrue: [ rootDEVS receiveMsgStarTime: clock ]].	 "If not at the end of simulation, the planned event will take place."	
		verbose ifTrue: [
			reportStream nextPutAll: '* Root DEVS Output Port Configuration: ', String cr.
		 	rootDEVS reportOutputPortConfig ]].  
	reportStream nextPutAll: '* Step duration: ', duration printString, ' ms', String cr; flush.

! !

!DEVSRootSolverRT methodsFor: 'asynchronous interrupts' stamp: 'vj 11/19/2004 02:00'!
stop

	rtTimer isNil ifFalse: [ rtTimer unschedule ].		"wake up"
	^ super stop.
! !

DEVSRootSolverRT subclass: #DEVnDESSRootSolverRT
	instanceVariableNames: 'tn h dh seDetected'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-RT'!

!DEVnDESSRootSolverRT methodsFor: 'as yet unclassified' stamp: 'vj 11/25/2004 15:09'!
simulationStep
	"Probably full of bugs."

	| inputOccured input msg  aSubModel interruptTime |

	"Synchronize clock with real time"
	rtTimer _ Timer at: ((startTime + ((clock min: stopTime) / rtFactor * 1000)) min: SmallInteger maxVal) rounded.
	rtTimer wait.		"Waiting can be interrupted by asynchronous internal or external event."

	"Accept potential interrupts"
	interruptTime _ self realTime min: clock. 
	inputOccured _ false.
	input _ Dictionary new.
	[ (msg _ inputQueue nextOrNil) == nil ] whileFalse: [ 
			input at: (rootDEVS inputPortNamed: msg first) put: msg second.
			inputOccured _ true ].
	[ (aSubModel _ seQueue nextOrNil) == nil ] whileFalse: [ 
			aSubModel sendMsgSETime: interruptTime ].
	inputOccured  ifTrue: [
			rootDEVS receiveMsgX: input time: interruptTime.	 "... put input & update event lists"
			verbose ifTrue: [
				reportStream nextPutAll: '* Root DEVS Input Port Configuration: ', String cr.			
				rootDEVS reportInputPortConfig ]].

	"Numeric step:"
	tn _ rootDEVS timeNext. 
	(clock < tn) & (clock < stopTime) ifTrue: [ ^ self numericStep ].	"Note: We do not consider numeric input."

	self assert: [ clock = tn ].

	"Discrete step:"
	reportStream nextPutAll: '****************** simulation time, real time: ', 
		clock asFloat printString, ', ', self realTime asFloat printString, String cr.
	tn <= stopTime ifTrue: [ 
		rootDEVS receiveMsgStarTime: clock. 				"send (*, t) to root DEVS"
		verbose ifTrue: [
			reportStream nextPutAll: '* Root DEVS Output Port Configuration: ', String cr.
	 		rootDEVS reportOutputPortConfig ]].
! !

!DEVnDESSRootSolverRT methodsFor: 'as yet unclassified' stamp: 'vj 11/25/2004 15:25'!
receiveMsgSETime: tnc
	"a pure copy of the same method form non-RT  DEVnDESSRootSolver"

	seDetected _ true.
	tnc < tn ifTrue: [ tn _ tnc ] ifFalse: [ (tnc closeTo: tn) ifFalse: [ self error: 'Bad sync !!'] ]

! !

!DEVnDESSRootSolverRT methodsFor: 'as yet unclassified' stamp: 'vj 11/25/2004 15:24'!
numericStep
	"a pure copy of the same method form non-RT  DEVnDESSRootSolver"

	clock + h < tn ifTrue: [ dh _ h ] ifFalse: [ dh _ tn - clock ].
	seDetected _ false.
	rootDEVS receiveNMsgStarTime: clock step: dh.		"send (*, t, h) to root DEVS"
	rootDEVS receiveNMsgX: {} time: clock step: dh.		"send (x, t, h) to root DEVS"
	reportStream nextPutAll: '.'.
	seDetected ifTrue: [ 	| dhmin dhmax | dhmin _ 0. dhmax _ dh.
		[ seDetected & ((dhmax - dhmin) < 0.001) ] whileFalse: [ 
			reportStream nextPutAll: ':'.
			seDetected ifTrue: [ dhmax _ dh ] ifFalse: [ dhmin _ dh ].
			dh _ (dhmax + dhmin)/2.
			seDetected _ false.
			rootDEVS receiveNMsgStarTime: clock step: dh.		
			rootDEVS receiveNMsgX: {} time: clock step: dh.
				"Note: rootDEVS and consequently an AtomicDEVnDESS is repeatedly called 
				 for the same time. It is supposed that it remembers state and time 
				 from which last step was made. It must check time for which it is called
				 and either revert to previous state or save current state before making a
				 step." ] ].
	clock _ clock + dh
! !

!DEVnDESSRootSolverRT methodsFor: 'as yet unclassified' stamp: 'vj 11/25/2004 15:25'!
initialize
	"a pure copy of the same method form non-RT  DEVnDESSRootSolver"

	super initialize.
	h _ 0.01.
! !

!DEVSRootSolverRT methodsFor: 'realtime simulation' stamp: 'vj 11/7/2004 15:58'!
realTime
	"returns real time in seconds"
	^ (Time millisecondClockValue - startTime)  * rtFactor / 1000  
! !

!DEVSRootSolverRT methodsFor: 'realtime simulation' stamp: 'vj 11/20/2004 16:38'!
prepareToStart

	startTime _ Time millisecondClockValue - (clock / rtFactor * 1000).
	rootDEVS receiveMsgInitTime: clock.
! !

!DEVSRootSolverRT methodsFor: 'realtime simulation' stamp: 'vj 11/19/2004 01:08'!
rtFactor: f

	self safelyDo: [ rtFactor _ f ]
! !

!DEVSRootSolverRT methodsFor: 'asynchronous interrupts' stamp: 'vj 8/23/2003 18:33'!
inject: value to: port

	inputQueue isNil ifFalse: [ inputQueue nextPut: { port. value. } ].
	rtTimer isNil ifFalse: [ rtTimer unschedule ].		"wake up"
! !

!DEVSRootSolverRT methodsFor: 'realtime simulation' stamp: 'vj 11/21/2004 02:50'!
simulate: t
	"Performs the simulation. For simulation in background use start/stop instead."

	startTime _ Time millisecondClockValue.
	stopTime _ t.	"in seconds"
	clock _ self realTime min: rootDEVS timeNext.		"isn't this redundant ???"
	self prepareToStart.
	startStopSemaphore initSignals.	"running simulation indication"
	[ clock <= stopTime ] whileTrue: [ self simulationStep ].
	startStopSemaphore signal.
	self prepareToStop.

	self finalize.	"stops processes in atomic submodels"

! !

!DEVSRootSolverRT methodsFor: 'realtime simulation' stamp: 'vj 11/15/2004 01:10'!
receiveMsgSETime: t
	"nothing to do here"

! !

!DEVSRootSolverRT methodsFor: 'asynchronous interrupts' stamp: 'vj 8/23/2003 18:33'!
stateEventIn: aSubmodel

	seQueue isNil ifFalse: [ seQueue nextPut: aSubmodel ].
	rtTimer isNil ifFalse: [ rtTimer unschedule ].		"wake up"
! !

!AbstractSimulation methodsFor: 'as yet unclassified' stamp: 'vj 11/20/2004 19:17'!
name: n
	name _ n! !

!AbstractSimulation methodsFor: 'private' stamp: 'vj 11/20/2004 19:40'!
initializeSimulationControl

	startStopSemaphore _ Semaphore new.
	startStopSemaphore signal.							"means that simulator has been requested to stop"
	idleSemaphore _ Semaphore new.
	idleSemaphore signal.								"means that simulator is stopped"
	accessProtect _ Semaphore forMutualExclusion.		"thread safe start/stop etc."
	process _ nil.
! !

!AbstractSimulation methodsFor: 'simulation control' stamp: 'vj 11/16/2004 14:39'!
isIdle

	^ idleSemaphore isSignaled! !

!AbstractSimulation methodsFor: 'copying' stamp: 'vj 11/20/2004 19:42'!
copy

	^ self shallowCopy initializeSimulationControl! !

!AbstractSimulation methodsFor: 'private' stamp: 'vj 11/16/2004 14:32'!
accessProtect

	^accessProtect! !

!AbstractSimulation methodsFor: 'simulation control' stamp: 'vj 11/19/2004 00:53'!
cancelStopTime

	self stopTime: Float infinity! !

!AbstractSimulation methodsFor: 'as yet unclassified' stamp: 'vj 11/21/2004 15:49'!
reportStream: s

	reportStream _ s
! !

!AbstractSimulation methodsFor: 'as yet unclassified' stamp: 'vj 11/20/2004 19:19'!
parent: p
	parent _ p! !

!AbstractSimulation methodsFor: 'simulation control' stamp: 'vj 11/20/2004 02:44'!
waitForStop

	idleSemaphore wait; signal.

	"DOES NOT WORK. WHY????"! !

!AbstractSimulation methodsFor: 'as yet unclassified' stamp: 'vj 11/20/2004 19:16'!
name
	^ name! !

!AbstractSimulation methodsFor: 'private' stamp: 'vj 11/19/2004 01:48'!
safelyDo: aBlock

	| r v |
	accessProtect critical: [
		r _ self stopUnprotected.
		v _ aBlock value.
		r ifTrue: [ self startUnprotected ] ].
	^ v
! !

!AbstractSimulation methodsFor: 'morph' stamp: 'vj 11/20/2004 03:08'!
openInWorld

	"TO DO"

	"constructors of morphs, the whole morph"

	"Buttons reset, start, stop"
	"Simulation time, stopTime"
	"Report"
! !

!AbstractSimulation methodsFor: 'simulation control' stamp: 'vj 11/19/2004 01:49'!
stop

	| r |
	accessProtect critical: [ r _ self stopUnprotected ].
	^ r
! !

!AbstractSimulation methodsFor: 'as yet unclassified' stamp: 'vj 11/21/2004 16:31'!
reportStream

	^ reportStream
! !

!AbstractSimulation methodsFor: 'private' stamp: 'vj 11/18/2004 00:59'!
processBody

	self prepareToStart.
	[ startStopSemaphore isSignaled ] whileFalse: [ 	
		clock <= stopTime
			ifTrue: [ self simulationStep ]
			ifFalse: [ startStopSemaphore signal ] ].
	self prepareToStop.
	idleSemaphore signal.
! !

!AbstractSimulation methodsFor: 'private' stamp: 'vj 11/15/2004 00:18'!
prepareToStart

	"Subclasses may define some initalization code."! !

!AbstractSimulation methodsFor: 'simulation control' stamp: 'vj 11/19/2004 01:05'!
stopTime: endTime

	self safelyDo: [ stopTime _ endTime ]. 				"in seconds"
! !

!AbstractSimulation methodsFor: 'private' stamp: 'vj 11/19/2004 01:48'!
stopUnprotected

	| r |
	r _ false.
	startStopSemaphore isSignaled ifFalse: [ 
		startStopSemaphore signal.	"A signal to stop."
		idleSemaphore wait; signal.	"Waiting for current step finishing." 
		r _ true ].
	^ r



! !

!AbstractSimulation methodsFor: 'private' stamp: 'vj 11/15/2004 00:20'!
prepareToStop

	"Subclasses may define some finalization code."! !

!AbstractSimulation methodsFor: 'private' stamp: 'vj 11/5/2004 02:33'!
simulationStep

	self subclassResponsibility
! !

!AbstractSimulation methodsFor: 'simulation control' stamp: 'vj 11/16/2004 16:43'!
restart

	self stop; finalize; initialize; start
! !

!AbstractSimulation methodsFor: 'simulation control' stamp: 'vj 11/16/2004 14:37'!
isRunning

	^ startStopSemaphore isSignaled not! !

!AbstractSimulation methodsFor: 'simple simulation' stamp: 'vj 11/21/2004 16:36'!
simulate: t

	stopTime _ t.
	self prepareToStart.
	startStopSemaphore initSignals.	"running simulation indication"
	[ clock < stopTime ] whileTrue: [ self simulationStep ].
	startStopSemaphore signal.
	self prepareToStop.

	reportStream flush.
! !

!AbstractSimulation methodsFor: 'private' stamp: 'vj 11/21/2004 15:46'!
initialize

	self initializeSimulationControl.
	clock _ 0. stopTime _ Float infinity.

	reportStream _ Transcript. verbose _ true.
! !

!AbstractSimulation methodsFor: 'private' stamp: 'vj 11/19/2004 01:47'!
startUnprotected

	| r |
	r _ true.
	startStopSemaphore isSignaled ifTrue: [ 
		idleSemaphore wait.		
		startStopSemaphore initSignals.	"Not signaled means the simulation may run."
		process := [ self processBody ] forkAt: Processor userBackgroundPriority.
		r _ false ].
	^ r
! !

Object subclass: #DEVSEntity
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Core'!

DEVSEntity subclass: #MyJob
	instanceVariableNames: 'n size'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Examples'!

!MyJob commentStamp: 'vj 7/21/2003 20:34' prior: 0!
Part of MySystem!

!MyJob class methodsFor: 'as yet unclassified' stamp: 'vj 7/20/2003 01:10'!
sizeBetween: sl and: sh

	^ self new size: (sl to: sh) atRandom.! !

!MyJob methodsFor: 'as yet unclassified' stamp: 'vj 7/22/2003 21:42'!
printOn: aStream

	aStream nextPutAll: 'aJob', (n isNil ifTrue: [''] ifFalse: ['(', n printString, ')']).! !

!MyJob methodsFor: 'as yet unclassified' stamp: 'vj 7/20/2003 01:08'!
size: s

	size _ s
! !

!MyJob methodsFor: 'as yet unclassified' stamp: 'vj 7/17/2003 22:18'!
size

	^ size! !

!MyJob methodsFor: 'as yet unclassified' stamp: 'vj 7/22/2003 21:16'!
n: num

	n _ num
! !

DEVSEntity subclass: #Port
	instanceVariableNames: 'inLines outLines hostDEVS'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Core'!

Port subclass: #InputPort
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Core'!

!Port methodsFor: 'as yet unclassified' stamp: 'vj 8/27/2003 00:55'!
hostDEVS

	^ hostDEVS! !

!Port methodsFor: 'as yet unclassified' stamp: 'vj 8/28/2003 18:06'!
inLines

	^ inLines! !

Port subclass: #OutputPort
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Core'!

Port subclass: #InputNPort
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Continuous'!

!Port methodsFor: 'as yet unclassified' stamp: 'vj 8/28/2003 18:06'!
outLines

	^ outLines! !

!Port methodsFor: 'as yet unclassified' stamp: 'vj 7/16/2003 17:21'!
hostDEVS: host

	hostDEVS _ host! !

!Port class methodsFor: 'as yet unclassified' stamp: 'vj 8/27/2003 15:27'!
hostDEVS: host name: n

	^ (super new initialize hostDEVS: host) name: n! !

!Port methodsFor: 'as yet unclassified' stamp: 'vj 8/28/2003 18:04'!
initialize

	inLines _ OrderedCollection new.
	outLines _ OrderedCollection new.
! !

Port subclass: #OutputNPort
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Continuous'!

!DEVSEntity methodsFor: 'as yet unclassified' stamp: 'vj 7/22/2003 21:11'!
printOn: aStream

	name isNil 
		ifTrue: [ 	| title |
			title _ self class name.
			aStream
				nextPutAll: (title first isVowel ifTrue: ['an'] ifFalse: ['a']); "without space"
				nextPutAll: title ]
		ifFalse: [ aStream nextPutAll: name asString ]
! !

DEVSEntity subclass: #BaseDEVS
	instanceVariableNames: 'parent inputPorts outputPorts input output timeLast timeNext verbose reportStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Core'!

!BaseDEVS methodsFor: 'modeling' stamp: 'vj 7/16/2003 20:43'!
inputPortNamed: n

	^ inputPorts at: n
	! !

!BaseDEVS methodsFor: 'port accessing' stamp: 'vj 7/26/2003 18:35'!
poke: x to: p

	output at: (self outputPortNamed: p) put: x! !

!BaseDEVS methodsFor: 'modeling2' stamp: 'vj 8/25/2003 20:59'!
removeOutputPorts: list

	list do: [ :p | self removeOutputPortNamed: p ].
! !

!BaseDEVS methodsFor: 'printing' stamp: 'vj 7/24/2003 16:44'!
reportOutputPortConfig

	outputPorts associationsDo: [ :assoc |
		reportStream nextPutAll: '        ', assoc key asString, ': ', String tab, (output at: assoc value ifAbsent: [nil]) printString, String cr ].! !

!BaseDEVS class methodsFor: 'as yet unclassified' stamp: 'vj 8/4/2003 19:45'!
getSimulatorRT

	^ self new getSimulatorRT
! !

!BaseDEVS methodsFor: 'copying' stamp: 'vj 9/20/2003 15:51'!
copy

	| aCopy |
	aCopy _ self shallowCopy.
	"Original ports are to be replaced by new ones."
	aCopy removeAllPorts.
	aCopy input: Dictionary new.
	aCopy output: Dictionary new.
	inputPorts keysAndValuesDo: [ :n :p || np | 
		np _ aCopy addInputPortOfType: p species named: n.
		(input keys includes: p) ifTrue: [ aCopy input at: np put: (input at: p) copy ]].	
	outputPorts keysAndValuesDo: [ :n :p || np |
		np _ aCopy addOutputPortOfType: p species named: n.
		(output keys includes: p) ifTrue: [ aCopy output at: np put: (output at: p) copy ]].
	^ aCopy
	
! !

!BaseDEVS methodsFor: 'modeling' stamp: 'vj 7/17/2003 01:52'!
outputPorts

	^ outputPorts values
! !

!BaseDEVS methodsFor: 'misc' stamp: 'vj 8/12/2003 00:30'!
input
	^input! !

!BaseDEVS methodsFor: 'simulation' stamp: 'vj 7/24/2003 17:01'!
reportStream: s

	reportStream _ s
! !

!BaseDEVS methodsFor: 'simulation' stamp: 'vj 7/21/2003 00:44'!
timeLast

	^ timeLast! !

!BaseDEVS methodsFor: 'modeling' stamp: 'vj 8/26/2003 22:53'!
addOutputPortNamed: n

	^ self addOutputPortOfType: OutputPort named: n! !

!BaseDEVS methodsFor: 'modeling' stamp: 'vj 7/24/2003 16:40'!
initialize

	inputPorts _ Dictionary new.
	outputPorts _ Dictionary new.
	input _ Dictionary new.
	output _ Dictionary new.

	verbose _ true.
	reportStream _ Transcript.


! !

!BaseDEVS methodsFor: 'simulation' stamp: 'vj 7/26/2003 18:15'!
verbose

	^ verbose 

! !

!BaseDEVS methodsFor: 'printing' stamp: 'vj 7/24/2003 16:44'!
reportInputPortConfig

	inputPorts associationsDo: [ :assoc |
		reportStream nextPutAll: '        ', assoc key asString, ':', String tab, (input at: assoc value ifAbsent: [nil]) printString, String cr ].! !

!BaseDEVS methodsFor: 'modeling' stamp: 'vj 8/26/2003 22:51'!
addInputPortNamed: n

	^ self addInputPortOfType: InputPort named: n! !

!BaseDEVS class methodsFor: 'as yet unclassified' stamp: 'vj 7/26/2003 18:19'!
getSimulator

	^ self new getSimulator
! !

BaseDEVS subclass: #CoupledDEVS
	instanceVariableNames: 'components namedComponents eventList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Core'!

!CoupledDEVS methodsFor: 'modeling' stamp: 'vj 7/21/2003 10:46'!
initialize

	super initialize.
	components _ OrderedCollection new.
	namedComponents _ Dictionary new.! !

CoupledDEVS subclass: #MySystem3b
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Examples'!

!MySystem3b commentStamp: 'vj 11/12/2003 23:05' prior: 0!
Similar to MySystems2. Uses MyGenerator2 instead of MyGenerator. Bulilt for RT simulation.

Example:
[ s _ MySystem3b getSimulatorRT. s simulate: 20. ] forkAt: Processor timingPriority.

!

!MySystem3b methodsFor: 'as yet unclassified' stamp: 'vj 8/23/2003 17:22'!
initialize

	super initialize.

	self addOutputPorts: {
		#out.
		#discard.
	}.

	self addComponents: {
		#generator -> (MyGenerator2 ia: 2 ib: 7 sa: 5 sb: 10).		"!! RT !!"
		#processor -> (MyProcessor withQueueSize: 5).
	}. 

	self addCouplings: { 
		#(generator out) -> #(processor in).
		#(processor out) -> #(self out).
		#(processor discard) -> #(self discard).
	}.



! !

!CoupledDEVS methodsFor: 'simulation' stamp: 'vj 7/21/2003 19:36'!
deepVerbose: truefalse

	self beVerbose: truefalse.
	components do: [ :c | c deepVerbose: truefalse ].
! !

!CoupledDEVS methodsFor: 'modeling2' stamp: 'vj 8/27/2003 02:22'!
addSubModel: model named: n

	model name: n.
	self addSubModel: model.
	namedComponents at: n put: model.	"namedComponents for easier model construction"

	^ model.! !

!CoupledDEVS methodsFor: 'modeling2' stamp: 'vj 8/30/2003 02:49'!
removeSubModelNamed: n

	| model |

	model _ namedComponents removeKey: n.
	self removeSubModel: model.
	model name: nil.

	^model! !

!CoupledDEVS methodsFor: 'simulation' stamp: 'vj 8/28/2003 23:20'!
reportStructure

	super reportStructure.
	components do: [ :c | c reportStructure ].


	! !

!CoupledDEVS methodsFor: 'modeling' stamp: 'vj 7/16/2003 17:47'!
select: immList

	^ immList at: 1! !

!CoupledDEVS methodsFor: 'modeling' stamp: 'vj 8/28/2003 18:08'!
unconnectPort: p1 from: p2

	p1 outLines remove: p2.
	p2 inLines remove: p1.
! !

!CoupledDEVS methodsFor: 'modeling2' stamp: 'vj 8/27/2003 00:00'!
addCouplings: net

	"aCoupledDEVS addCouplings: {
		#(model1 port2) -> #(model3 port4). 
		#(model5 port6) -> #(model7 port8). ... }"

	net do: [ :connection |
		self connectSubModelNamed: connection key first port: connection key second
			toSubModelNamed: connection value first port: connection value second ].

! !

!CoupledDEVS methodsFor: 'modeling' stamp: 'vj 11/16/2004 13:51'!
removeSubModel: model

	self safelyDo: [ 
		components remove: model.
		model parent: nil ].
	^ model

! !

CoupledDEVS subclass: #MySystem2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Examples'!

!MySystem2 commentStamp: 'vj 11/12/2003 22:56' prior: 0!
DEVS example. A demonstartion of another way of coupled model specification. See initialize. Log goes to Transcript.

Example:
MySystem2 getSimulator simulate: 100.

!

!MySystem2 methodsFor: 'as yet unclassified' stamp: 'vj 8/23/2003 17:21'!
initialize

	super initialize.

	self addOutputPorts: {
		#out.
		#discard.
	}.

	self addComponents: {
		#generator -> (MyGenerator ia: 2 ib: 7 sa: 5 sb: 10).
		#processor -> (MyProcessor withQueueSize: 5).
	}. 

	self addCouplings: { 
		#(generator out) 	-> #(processor in).
		#(processor out) 	-> #(self out).
		#(processor discard) -> #(self discard).
	}.



! !

MySystem2 subclass: #MySystem3
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Examples'!

!MySystem3 commentStamp: 'vj 11/12/2003 23:04' prior: 0!
DEVS example. A demonstartion of realtime simulation.

Example:
| s |
[ s _ MySystem3 getSimulatorRT. s simulate: 10. ] forkAt: Processor userBackgroundPriority.
(Delay forSeconds: 3) wait.
s inject: #aaaa to: #in
!

!MySystem3 methodsFor: 'as yet unclassified' stamp: 'vj 7/27/2003 03:03'!
initialize

	super initialize.

	self addInputPorts: {
		#in.
	}.

! !

!CoupledDEVS methodsFor: 'modeling' stamp: 'vj 11/16/2004 13:53'!
addSubModel: model

	self safelyDo: [
		components add: model.
		model parent: self ].
	^ model! !

!CoupledDEVS methodsFor: 'modeling2' stamp: 'vj 8/25/2003 21:25'!
removeCouplings: net

	"aCoupledDEVS removeCouplings: {
		(#model1 #port2) -> (#model3 #port4). 
		(#model5 #port6) -> (#model7 #port8). ... }"

	net do: [ :connection |
		self unconnectSubModelNamed: connection key first port: connection key second
			fromSubModelNamed: connection value first port: connection value second ].

! !

!CoupledDEVS methodsFor: 'copying' stamp: 'vj 8/30/2003 17:36'!
copy

	| aCopy |

	aCopy _ super copy.	 "ports were replaced by new ones, without links (w/o couplings)"
	aCopy removeAllSubModels.
	(components difference: namedComponents values) do: [ :c |
		aCopy addSubModel: c copy ].
	namedComponents keysAndValuesDo: [ :k :v |
		aCopy addSubModel: v copy named: k ].
	aCopy addCouplings: self couplings.	

	^  aCopy! !

!CoupledDEVS methodsFor: 'modeling' stamp: 'vj 8/26/2003 23:52'!
removeAllSubModels

	components _ OrderedCollection new.
	namedComponents _ Dictionary new.! !

!CoupledDEVS methodsFor: 'modeling2' stamp: 'vj 7/21/2003 17:10'!
subModelNamed: n

	n == #self ifTrue: [ ^ self ].
	^ namedComponents at: n! !

!CoupledDEVS methodsFor: 'simulation' stamp: 'vj 8/23/2003 22:29'!
finalize

	components do: [ :c | c finalize ]! !

CoupledDEVS subclass: #MySystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Examples'!

!MySystem commentStamp: 'vj 11/12/2003 22:56' prior: 0!
DEVS example. Log goes to Transcript.

Example:
(MySystem getSimulator deepVerbose: true) simulate: 100.

!

!MySystem methodsFor: 'as yet unclassified' stamp: 'vj 7/22/2003 22:00'!
initialize

	| out discard generator prevPort tmp |
	super initialize.

	out _ self addOutputPortNamed: #out.
	discard _ self addOutputPortNamed: #discard.

	generator _ self addSubModel: (MyGenerator ia: 2 ib: 7 sa: 5 sb: 10) named: #generator.

	prevPort _ generator outputPortNamed: #out.
	1 to: 3 do: [ :i |
		tmp _ self addSubModel: (MyProcessor withQueueSize: 5) named: 
										(#processor, i printString) asSymbol.
		self connectPort: prevPort to: (tmp inputPortNamed: #in).
		self connectPort: (tmp outputPortNamed: #out) to: out.
		prevPort _ tmp outputPortNamed: #discard ].
	self connectPort: prevPort to: discard.



! !

CoupledDEVS subclass: #CoupledDEVnDESS
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Continuous'!

CoupledDEVnDESS subclass: #MyContinuousSystem3
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Examples'!

!MyContinuousSystem3 commentStamp: 'vj 11/12/2003 23:15' prior: 0!
Coupled DEVnDESS example.!

!MyContinuousSystem3 methodsFor: 'as yet unclassified' stamp: 'vj 8/31/2003 01:11'!
initialize

	super initialize.


	self addComponents: {
		#intg1 	-> (Integrator newWithInitialCondition: -0.5).
		#intg2 	-> (Integrator newWithInitialCondition: 0.5).
		#intg3 	-> (Integrator newWithInitialCondition: 0.3).
		#f1 	-> (InstantaneousFunction withInputs: {#i1. #i2.} codeBlock: [:i1 :i2 | 
					("a" 0.2 * i1) + i2 ]).
		#f2 	-> (InstantaneousFunction withInputs: {#i1. #i2.} codeBlock: [:i1 :i2 | 
					0 - i1 - i2 ]).
		#f3 	-> (InstantaneousFunction withInputs: {#i1. #i2.} codeBlock: [:i1 :i2 | 
					0.5 + (i1 - 0.3) * i2 ]).
		#plotter -> (Plotter withInputs: {#i1. #i2. #i3}).
	}. 

	self addCouplings: { 
		#(f1 output) 	-> #(intg1 input).
		#(f2 output) 	-> #(intg2 input).
		#(f3 output) 	-> #(intg3 input).
		#(intg1 output)	-> #(f1 i1).
		#(intg1 output)	-> #(f2 i1).
		#(intg2 output) 	-> #(f1 i2).
		#(intg2 output) 	-> #(f3 i1).
		#(intg3 output) 	-> #(f2 i2).
		#(intg3 output) 	-> #(f3 i2).
		#(intg1 output) 	-> #(plotter i1).
		#(intg2 output)	-> #(plotter i2).
		#(intg3 output)	-> #(plotter i3).
	}.

! !

!CoupledDEVnDESS methodsFor: 'as yet unclassified' stamp: 'vj 8/30/2003 17:24'!
receiveNMsgStarTime: t step: h

	| yy xx yetUnevaluatedFunctions toEvaluate |

	"get output from (send '*' to) all blocks with integrators and collect their outputs"
	yy _ Dictionary new.	
	(components select: [ :c | (c isKindOf: CoupledDEVnDESS) | (c isKindOf: AtomicDEVnDESS) ]) 
		do: [ :c | (c receiveNMsgStarTime: t step: h) associationsDo: [ :a | yy add: a ] ].

	"evaluate memoryless fast blocks with defined inputs while it is possible 
	 (each block is evaluated max 1 time)"
	yetUnevaluatedFunctions _ components select: [ :c | (c isKindOf: InstantaneousFunction) ].
	toEvaluate _ OrderedCollection new.
	[ toEvaluate size = yetUnevaluatedFunctions size ] whileFalse: [ 
	  toEvaluate _ yetUnevaluatedFunctions copy.
	  toEvaluate do: [ :c | | ready |	
		xx _ Dictionary new. ready _ true.
		(c inputPorts select: [ :p | p isKindOf: InputNPort]) do: [ :p | p inLines do: [ :pi |	
			(yy keys includes: pi) ifTrue: [ xx at: p put: (yy at: pi) ] ifFalse: [ ready _ false ]]].
		ready ifTrue: [ 
			(c receiveNMsgX: xx time: t step: h) associationsDo: [ :a | yy add: a ]. 
			yetUnevaluatedFunctions remove: c ]]].

	"define output of self"
	output _ Dictionary new.
	(outputPorts select: [ :p | p isKindOf: OutputNPort]) do: [ :p | p inLines do: [ :pi |
		(yy keys includes: pi) ifTrue: [ output at: p put: (yy at: pi) ]]].

	^ output
! !

!CoupledDEVnDESS methodsFor: 'as yet unclassified' stamp: 'vj 8/26/2003 22:58'!
addInputNPortNamed: n

	^ self addInputPortOfType: InputNPort named: n! !

!CoupledDEVnDESS methodsFor: 'as yet unclassified' stamp: 'vj 8/26/2003 22:59'!
addOutputNPortNamed: n

	^ self addOutputPortOfType: OutputNPort named: n! !

!CoupledDEVnDESS methodsFor: 'as yet unclassified' stamp: 'vj 11/24/2004 16:48'!
getSimulatorRT

	^ (DEVnDESSRootSolverRT basicNew rootDEVS: self) initialize! !

!CoupledDEVnDESS methodsFor: 'as yet unclassified' stamp: 'vj 8/28/2003 18:09'!
receiveNMsgX: x time: t step: h

	| xx yy |

	(timeLast <= t) & (t <= timeNext) ifFalse: [ self error: 'Bad Synchronization' ].

	input _ x.

	"collect previously generated outputs from all numeric components, including self's input"
	yy _ Dictionary new.
	input associationsDo: [ :a | (a key isKindOf: InputNPort) ifTrue: [ yy add: a ] ].
	(components select: [ :c | (c isKindOf: CoupledDEVnDESS) | (c isKindOf: AtomicDEVnDESS) | 
	   (c isKindOf: InstantaneousFunction) ]) 
		do: [ :c | c output associationsDo: [ :a | 
			(a key isKindOf: OutputNPort) ifTrue: [ yy add: a ] ] ].

	"evaluate all numeric components with appropriate inputs - make integration step"
	(components select: [ :c | (c isKindOf: CoupledDEVnDESS) | (c isKindOf: AtomicDEVnDESS) ]) 
	  do: [ :c |
		xx _ Dictionary new.
		(c inputPorts select: [ :p | p isKindOf: InputNPort]) do: [ :p | p inLines do: [ :pi |
			(yy keys includes: pi) ifTrue: [ xx at: p put: (yy at: pi) ]]].
		c receiveNMsgX: xx time: t step: h ].

! !

CoupledDEVnDESS subclass: #MyContinuousSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Examples'!

!MyContinuousSystem commentStamp: 'vj 11/12/2003 23:15' prior: 0!
Coupled DEVnDESS example.!

!MyContinuousSystem methodsFor: 'as yet unclassified' stamp: 'vj 9/13/2003 18:46'!
initialize

	super initialize.

	self addComponents: {
		#contiblock 	-> (MyContinuousBlock new).
		#plotter 	-> (Plotter withInputs: {#i1. #i2.}).
		#plotter2	-> (XYPlotter new)
	}. 

	self addCouplings: { 
		#(contiblock o1) -> #(plotter i1).
		#(contiblock o2) -> #(plotter i2).
		#(contiblock o1) -> #(plotter2 x).
		#(contiblock o2) -> #(plotter2 y).
	}.
! !

CoupledDEVnDESS subclass: #MySystems
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Examples'!

!MySystems commentStamp: 'vj 11/12/2003 23:03' prior: 0!
DEVS example. All systems coupled together. MyDaemon (part of MySystems) modifies structure of MySystems at runtime!

!MySystems methodsFor: 'as yet unclassified' stamp: 'vj 8/27/2003 03:29'!
initialize

	super initialize.

	self addComponents: {
		#s1 -> (MySystem new).
		#s2 -> (MySystem2 new).
		#c1 -> (MyContinuousSystem new).
		#c2 -> (MyContinuousSystem2 new).
		#d	-> (MyDaemon new)
	}. 
! !

!CoupledDEVnDESS methodsFor: 'as yet unclassified' stamp: 'vj 3/17/2004 23:29'!
getSimulator

	^ (DEVnDESSRootSolver basicNew rootDEVS: self) initialize! !

CoupledDEVnDESS subclass: #MyContinuousSystem2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Examples'!

!MyContinuousSystem2 commentStamp: 'vj 11/12/2003 23:15' prior: 0!
Coupled DEVnDESS example.!

!MyContinuousSystem2 methodsFor: 'as yet unclassified' stamp: 'vj 8/23/2003 17:17'!
initialize

	super initialize.


	self addComponents: {
		#intg1 	-> (Integrator newWithInitialCondition: -2.0).
		#intg2 	-> (Integrator newWithInitialCondition: 2.0).
		#f 		-> (InstantaneousFunction withInputs: {#i1. #i2.} codeBlock: [:i1 :i2 | 
					1.0 * (1.0 - i1 squared) * i2 - i1 ]).
		#plotter -> (Plotter withInputs: {#i1. #i2.}).
	}. 

	self addCouplings: { 
		#(intg1 output)	-> #(f i1).
		#(intg2 output) 	-> #(f i2).
		#(f output) 		-> #(intg2 input).
		#(intg2 output) 	-> #(intg1 input).
		#(intg1 output) 	-> #(plotter i1).
		#(intg2 output)	-> #(plotter i2).
	}.

! !

CoupledDEVnDESS subclass: #MyContinuousSystem1
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Examples'!

!MyContinuousSystem1 commentStamp: 'vj 11/12/2003 23:15' prior: 0!
Coupled DEVnDESS example.!

!MyContinuousSystem1 methodsFor: 'as yet unclassified' stamp: 'vj 9/13/2003 20:34'!
initialize

	super initialize.

	self addComponents: {
		#contiblock 	-> (MyContinuousBlock2 new).
		#plotter 	-> (Plotter withInputs: {#i1. #i2.}).
		#plotter2	-> (XYPlotter new)
	}. 

	self addCouplings: { 
		#(contiblock o1) -> #(plotter i1).
		#(contiblock o2) -> #(plotter i2).
		#(contiblock o1) -> #(plotter2 x).
		#(contiblock o2) -> #(plotter2 y).
	}.
! !

!CoupledDEVS methodsFor: 'modeling2' stamp: 'vj 7/21/2003 18:54'!
addComponents: compList

	compList do: [ :assoc |
		self addSubModel: (assoc value) named: (assoc key) ].
! !

!CoupledDEVS methodsFor: 'modeling2' stamp: 'vj 8/28/2003 18:09'!
couplings

	"Returns something like this:
	 {	#(model1 port2) -> #(model3 port4). 
		#(model5 port6) -> #(model7 port8). ... }"
	"Unfortunately, ports has not been designed effectively with respect to
	 introspection and reflectivity. So we have to do hard work here."

	| couplingSpecification |

	couplingSpecification _ OrderedCollection new.
	self inputPorts do: [ :p | p outLines do: [ :l |
		couplingSpecification add: ({#self. p name} -> {l hostDEVS. l name}) ]].
	components do: [ :m | m outputPorts do: [ :p | p outLines do: [ :l |
		couplingSpecification add: 
			({m name. p name} -> 
			{l hostDEVS = self ifTrue: [#self] ifFalse: [l hostDEVS name]. l name}) ]]].
	^ couplingSpecification asArray
! !

!CoupledDEVS methodsFor: 'modeling2' stamp: 'vj 8/25/2003 21:23'!
unconnectSubModelNamed: c1 port: p1 fromSubModelNamed: c2 port: p2

	| port1 port2 | 

	port1 _ c1 = #self ifFalse: [ (self subModelNamed: c1) outputPortNamed: p1 ] 
						ifTrue: [ self inputPortNamed: p1 ].
	port2 _ c2 = #self ifFalse: [ (self subModelNamed: c2) inputPortNamed: p2 ] 
						ifTrue: [ self outputPortNamed: p2 ].

	self unconnectPort: port1 from: port2.! !

!CoupledDEVS methodsFor: 'simulation' stamp: 'vj 8/23/2003 19:05'!
receiveMsgSETime: t

	t < timeNext ifTrue: [ 
		timeNext _ t.
		eventList _ SortedCollection sortBlock: [ :a :b |  (a at: 1) <= (b at: 1) ].
		components do: [ :c | eventList add: { c timeNext. c } ].
		parent receiveMsgSETime: t ].


! !

!CoupledDEVS methodsFor: 'simulation' stamp: 'vj 12/5/2004 00:15'!
receiveMsgX: x time: t

	| xx |

	(timeLast <= t) & (t <= timeNext) ifFalse: [ self error: 'Bad Synchronization' ].

	input _ x.

	timeLast _ t.
	timeNext _ Float infinity.
	eventList _ SortedCollection sortBlock: [ :a :b |  (a at: 1) <= (b at: 1) ].
	components do: [ :c |
		xx _ Dictionary new.
		c inputPorts do: [ :p | p inLines do: [ :pi |
			(input keys includes: pi) ifTrue: [ xx at: p put: (input at: pi) ]]].
		xx isEmpty ifFalse: [ c receiveMsgX: xx time: t ].
		eventList add: { c timeNext. c. }.
		timeNext _ timeNext min: (c timeNext) ].


! !

!CoupledDEVS methodsFor: 'simulation' stamp: 'vj 12/5/2004 00:15'!
receiveMsgStarTime: t

	| immChildren selected yy xx |

	(t closeTo: timeNext) ifFalse: [ self error: 'Bad synchronization' ].

	immChildren _ OrderedCollection new.
	eventList do: [ :ev |
		((ev at: 1) closeTo: t) ifTrue: [ immChildren add: (ev at: 2) ]].
	immChildren size = 1 
		ifTrue: [ selected _ immChildren at: 1 ]
		ifFalse: [ selected _ self select: immChildren.
				verbose ifTrue: [
					reportStream nextPutAll: '* Collision occured in ', self printString, ', involving: ', 
						immChildren printString, String cr.
					reportStream nextPutAll: '        select chooses: ', selected printString, String cr] ].

	yy _ selected receiveMsgStarTime: t.

	timeLast _ t.
	timeNext _ Float infinity.
	eventList _ SortedCollection sortBlock: [ :a :b |  (a at: 1) <= (b at: 1) ].
	components do: [ :c | 
		xx _ Dictionary new.
		c inputPorts do: [ :p | p inLines do: [ :pi |	
			(yy keys includes: pi) ifTrue: [ xx at: p put: (yy at: pi) ]]].
		xx isEmpty ifFalse: [ c receiveMsgX: xx time: t ].
		eventList add: { c timeNext. c. }.
		timeNext _ timeNext min: (c timeNext) ].

	output _ Dictionary new.
	outputPorts do: [ :p | p inLines do: [ :pi |
		(yy keys includes: pi) ifTrue: [ output at: p put: (yy at: pi) ]]].

	^ output

! !

!CoupledDEVS methodsFor: 'modeling2' stamp: 'vj 8/25/2003 21:17'!
removeComponents: compList

	compList do: [ :x |
		self removeSubModelNamed: x ].
! !

!CoupledDEVS methodsFor: 'modeling2' stamp: 'vj 11/6/2004 01:44'!
componentNames

	^ namedComponents keys! !

!CoupledDEVS methodsFor: 'copying' stamp: 'vj 8/28/2003 02:23'!
storeOn: aStream

	aStream nextPut: $(.
	aStream nextPutAll: self class name, ' basicNew'.
	(self class allInstVarNames difference: {'parent'. 'inputPorts'. 'outputPorts'. 'input'. 'output'.
		'reportStream'. 'components'. 'namedComponents'. 'eventList'}) do:
		[:n |
		aStream nextPutAll: ' instVarNamed: ';
			store: n;
			nextPutAll: ' put: ';
			store: (self instVarNamed: n);
			nextPut: $;].

	"ports"
	aStream nextPutAll: ' removeAllPorts;'.
	inputPorts keysAndValuesDo: [ :k :v | 
		aStream nextPutAll: ' addInputPortOfType: '; store: v species; nextPutAll: ' named: '; store: k; nextPut: $; ].	
	outputPorts keysAndValuesDo: [ :k :v | 
		aStream nextPutAll: ' addOutputPortOfType: '; store: v species; nextPutAll: ' named: '; store: k; nextPut: $; ].	
	"components"
	aStream nextPutAll: ' removeAllSubModels;'.
	(components difference: namedComponents values) do: [ :c |
		aStream nextPutAll: 'addSubModel: '; store: c; nextPut: $; ].
	namedComponents keysAndValuesDo: [ :k :v |
		aStream nextPutAll: 'addSubModel: '; store: v; nextPutAll: ' named: '; store: k; nextPut: $; ].
	"couplings"
	aStream nextPutAll: ' addCouplings: '; store: self couplings; nextPut: $;.	

	aStream nextPutAll: ' yourself)'
! !

!CoupledDEVS methodsFor: 'simulation' stamp: 'vj 12/5/2004 00:08'!
receiveMsgInitTime: t

	timeLast _ 0.
	timeNext _ Float infinity.
	eventList _ SortedCollection sortBlock: [ :a :b |  (a at: 1) <= (b at: 1) ].
	components do: [ :c |
		c receiveMsgInitTime: t.
		eventList add: { c timeNext. c }.
		timeNext _ timeNext min: c timeNext.
		timeLast _ timeLast max: c timeLast ].
! !

!CoupledDEVS methodsFor: 'modeling' stamp: 'vj 8/28/2003 18:07'!
connectPort: p1 to: p2

	p1 outLines add: p2.
	p2 inLines add: p1.
! !

!CoupledDEVS methodsFor: 'simulation' stamp: 'vj 7/26/2003 17:23'!
reportStream: s

	reportStream _ s.
	components do: [ :c | c reportStream: s ]! !

!CoupledDEVS methodsFor: 'simulation' stamp: 'vj 8/30/2003 17:17'!
receiveMsgSync

	components do: [ :c | c receiveMsgSync ].
! !

!CoupledDEVS methodsFor: 'modeling2' stamp: 'vj 7/21/2003 17:22'!
connectSubModelNamed: c1 port: p1 toSubModelNamed: c2 port: p2

	| port1 port2 | 

	port1 _ c1 = #self ifFalse: [ (self subModelNamed: c1) outputPortNamed: p1 ] 
						ifTrue: [ self inputPortNamed: p1 ].
	port2 _ c2 = #self ifFalse: [ (self subModelNamed: c2) inputPortNamed: p2 ] 
						ifTrue: [ self outputPortNamed: p2 ].

	self connectPort: port1 to: port2.! !

!BaseDEVS methodsFor: 'simulation' stamp: 'vj 7/21/2003 00:44'!
timeNext

	^ timeNext! !

!BaseDEVS methodsFor: 'modeling2' stamp: 'vj 8/25/2003 20:59'!
removeInputPorts: list

	list do: [ :p | self removeInputPortNamed: p ].
! !

!BaseDEVS methodsFor: 'modeling2' stamp: 'vj 7/21/2003 18:59'!
addOutputPorts: list

	list do: [ :p | self addOutputPortNamed: p ].
! !

!BaseDEVS methodsFor: 'modeling' stamp: 'vj 7/17/2003 01:52'!
inputPorts

	^ inputPorts values
! !

!BaseDEVS methodsFor: 'private' stamp: 'vj 8/27/2003 15:28'!
addOutputPortOfType: portClass named: n

	| port |
	port _ portClass hostDEVS: self name: n.
	outputPorts at: n put: port.
	^ port! !

!BaseDEVS methodsFor: 'printing' stamp: 'vj 8/1/2003 21:59'!
printOn: aStream

	(parent isKindOf: BaseDEVS) ifTrue: [ parent printOn: aStream. aStream nextPutAll: '/' ].
	super printOn: aStream.

! !

!BaseDEVS methodsFor: 'simulation' stamp: 'vj 7/21/2003 19:21'!
beVerbose: truefalse

	verbose _ truefalse.

! !

!BaseDEVS methodsFor: 'simulation' stamp: 'vj 7/21/2003 19:28'!
deepVerbose: truefalse

	self beVerbose: truefalse.

! !

!BaseDEVS methodsFor: 'simulation' stamp: 'vj 11/6/2004 18:52'!
getSimulatorRT

	^ (DEVSRootSolverRT basicNew rootDEVS: self) initialize! !

!BaseDEVS methodsFor: 'modeling' stamp: 'vj 8/2/2003 01:43'!
parent

	^ parent
! !

!BaseDEVS methodsFor: 'simulation' stamp: 'vj 3/17/2004 23:28'!
getSimulator

	^ (DEVSRootSolver basicNew rootDEVS: self) initialize! !

!BaseDEVS methodsFor: 'misc' stamp: 'vj 9/20/2003 15:05'!
output: o
	output _ o! !

!BaseDEVS methodsFor: 'modeling' stamp: 'vj 8/25/2003 20:57'!
removeInputPortNamed: n

	inputPorts removeKey: n.
! !

!BaseDEVS methodsFor: 'modeling2' stamp: 'vj 7/21/2003 18:58'!
addInputPorts: list

	list do: [ :p | self addInputPortNamed: p ].
! !

BaseDEVS subclass: #AtomicDEVS
	instanceVariableNames: 'elapsed savedTimeAdvance'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Core'!

!AtomicDEVS methodsFor: 'simulation' stamp: 'vj 8/23/2003 18:58'!
sendMsgSETime: t

	parent receiveMsgSETime: (timeNext _ t).
! !

!AtomicDEVS methodsFor: 'modeling' stamp: 'vj 8/4/2003 18:37'!
extTransition

	self subclassResponsibility! !

!AtomicDEVS methodsFor: 'modeling' stamp: 'vj 11/3/2004 18:28'!
elapsed
	^elapsed! !

!AtomicDEVS methodsFor: 'simulation' stamp: 'vj 11/16/2004 15:58'!
receiveMsgInitTime: t

	timeLast _ t - elapsed.
	savedTimeAdvance isNil ifTrue: [ savedTimeAdvance _ self timeAdvance ].
	timeNext _ timeLast + savedTimeAdvance.
	timeNext < t ifTrue: [ self error: 'Bad Sync !!!!!!' ].

	self prepareToStart. 		"resume processes in atomic model"
! !

AtomicDEVS subclass: #AtomicDEVnDESS
	instanceVariableNames: 'time step intg intgTmp savedIntg conditions savedConditionValues sePolarity seConditionIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Continuous'!

AtomicDEVnDESS subclass: #MyContinuousBlock3
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Examples'!

!MyContinuousBlock3 commentStamp: 'vj 11/12/2003 23:14' prior: 0!
Atomic DEVnDESS example.!

!MyContinuousBlock3 methodsFor: 'as yet unclassified' stamp: 'vj 9/1/2003 21:37'!
outputFnc

! !

!MyContinuousBlock3 methodsFor: 'as yet unclassified' stamp: 'vj 9/1/2003 21:51'!
intgStep

	self intg: 1 integrate: [ 10 * ((intg at: 2) - (intg at: 1)) ].
 	self intg: 2 integrate: [ (1 + 24 - (intg at: 3)) * (intg at: 1) - (intg at: 2) ].
	self intg: 3 integrate: [ ((intg at: 1) * (intg at: 2)) - (2 * (intg at: 3)) ].

! !

!MyContinuousBlock3 methodsFor: 'as yet unclassified' stamp: 'vj 12/5/2004 00:18'!
timeAdvance

	^ Float infinity
! !

!MyContinuousBlock3 methodsFor: 'as yet unclassified' stamp: 'vj 9/1/2003 21:57'!
initConditions

	self allocateIntegrators: 3.

	intg at: 1 put: 0.1.
	intg at: 2 put: 0.2.
	intg at: 3 put: 0.3.

	self addOutputNPortNamed: #o1.
	self addOutputNPortNamed: #o2.
	self addOutputNPortNamed: #o3.

! !

!MyContinuousBlock3 methodsFor: 'as yet unclassified' stamp: 'vj 9/1/2003 21:31'!
intTransition

! !

!MyContinuousBlock3 methodsFor: 'as yet unclassified' stamp: 'vj 9/1/2003 21:37'!
outputNFnc

	self poke: (intg at: 1) to: #o1.
	self poke: (intg at: 2) to: #o2.
	self poke: (intg at: 3) to: #o3.

! !

!AtomicDEVnDESS methodsFor: 'as yet unclassified' stamp: 'vj 8/5/2003 22:00'!
initConditions

	self subclassResponsibility! !

!AtomicDEVnDESS methodsFor: 'as yet unclassified' stamp: 'vj 3/17/2004 23:28'!
getSimulator

	^ (DEVnDESSRootSolver basicNew rootDEVS: self) initialize! !

!AtomicDEVnDESS methodsFor: 'as yet unclassified' stamp: 'vj 8/15/2003 01:58'!
addSECondition: aBlock

	conditions add: aBlock.
	savedConditionValues _ nil. "it will be lazily generated later"

! !

!AtomicDEVnDESS methodsFor: 'as yet unclassified' stamp: 'vj 8/5/2003 21:35'!
intgPostStep

	intg _ intgTmp copy.
! !

!AtomicDEVnDESS methodsFor: 'as yet unclassified' stamp: 'vj 8/13/2003 19:32'!
saveState

	savedIntg _ intg copy.

	savedConditionValues isNil ifTrue: [ savedConditionValues _ Array new: conditions size ].
	1 to: conditions size do: [ :i | savedConditionValues at: i put: (conditions at: i) value ].! !

!AtomicDEVnDESS methodsFor: 'as yet unclassified' stamp: 'vj 8/26/2003 22:57'!
addInputNPortNamed: n

	^ self addInputPortOfType: InputNPort named: n! !

!AtomicDEVnDESS methodsFor: 'as yet unclassified' stamp: 'vj 8/25/2003 21:51'!
removeSEConditionNo: index

	conditions remove: (conditions at: index).
	savedConditionValues _ nil. "it will be lazily generated later"

! !

AtomicDEVnDESS subclass: #Integrator
	instanceVariableNames: 'initCondition pm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Continuous'!

!Integrator methodsFor: 'as yet unclassified' stamp: 'vj 8/25/2003 21:59'!
initCondition: aValue

	initCondition _ aValue.
	intg at: 1 put: initCondition.
	self saveState.


! !

!Integrator class methodsFor: 'as yet unclassified' stamp: 'vj 8/12/2003 03:46'!
newWithInitialCondition: aValue

	| integrator |
	integrator _ self new.
	integrator addInputNPortNamed: #input; addOutputNPortNamed: #output.	
	integrator initCondition: aValue.
	^ integrator! !

!Integrator methodsFor: 'as yet unclassified' stamp: 'vj 8/13/2003 17:02'!
initConditions

	self allocateIntegrators: 1.
! !

!Integrator methodsFor: 'as yet unclassified' stamp: 'vj 12/5/2004 00:16'!
timeAdvance

	^ Float infinity

! !

!Integrator methodsFor: 'as yet unclassified' stamp: 'vj 8/13/2003 02:03'!
outputNFnc

	self poke: (intg at: 1) to: #output.

! !

!Integrator methodsFor: 'as yet unclassified' stamp: 'vj 8/12/2003 02:44'!
intgStep

	self intg: 1 integrate: [ input values at: 1 ].

! !

!AtomicDEVnDESS methodsFor: 'as yet unclassified' stamp: 'vj 11/24/2004 16:48'!
getSimulatorRT

	^ (DEVnDESSRootSolverRT basicNew rootDEVS: self) initialize! !

AtomicDEVnDESS subclass: #MyContinuousBlock4
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Examples'!

!MyContinuousBlock4 commentStamp: 'vj 11/12/2003 23:14' prior: 0!
Atomic DEVnDESS example.!

!MyContinuousBlock4 methodsFor: 'as yet unclassified' stamp: 'vj 9/1/2003 22:06'!
intgStep

	self intg: 1 integrate: [ intg at: 2 ].
 	self intg: 2 integrate: [ intg at: 3 ].
	self intg: 3 integrate: [ -0.4 * (intg at: 3) - (intg at: 2) + (0.65 * (intg at: 1) * (1.0 - (intg at: 1) squared)) ].

! !

!MyContinuousBlock4 methodsFor: 'as yet unclassified' stamp: 'vj 9/1/2003 21:59'!
intTransition

! !

!MyContinuousBlock4 methodsFor: 'as yet unclassified' stamp: 'vj 9/1/2003 21:59'!
outputNFnc

	self poke: (intg at: 1) to: #o1.
	self poke: (intg at: 2) to: #o2.
	self poke: (intg at: 3) to: #o3.

! !

!MyContinuousBlock4 methodsFor: 'as yet unclassified' stamp: 'vj 9/1/2003 21:59'!
outputFnc

! !

!MyContinuousBlock4 methodsFor: 'as yet unclassified' stamp: 'vj 9/1/2003 21:59'!
initConditions

	self allocateIntegrators: 3.

	intg at: 1 put: 0.1234.
	intg at: 2 put: 0.2.
	intg at: 3 put: 0.1.

	self addOutputNPortNamed: #o1.
	self addOutputNPortNamed: #o2.
	self addOutputNPortNamed: #o3.

! !

!MyContinuousBlock4 methodsFor: 'as yet unclassified' stamp: 'vj 12/5/2004 00:19'!
timeAdvance

	^ Float infinity
! !

!AtomicDEVnDESS methodsFor: 'as yet unclassified' stamp: 'vj 9/13/2003 22:32'!
receiveNMsgStarTime: t step: h

	(t = time) "no #closeTo: because step restart is initated by exatly the same argument (?)"
		ifTrue: [ 
			self revertToPrevState ]
		ifFalse: [ 
			(t closeTo: time + step) ifFalse: [ self error: 'Bad sync !!!!!!']. 
			self saveState ].

	"generate output"
	time _ t.
	step _ h.
	output _ Dictionary new.
	self outputNFnc.
	^ output

! !

AtomicDEVnDESS subclass: #MyContinuousBlock2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Examples'!

!MyContinuousBlock2 commentStamp: 'vj 11/12/2003 23:18' prior: 0!
Atomic DEVnDESS example.!

!MyContinuousBlock2 methodsFor: 'as yet unclassified' stamp: 'vj 12/5/2004 00:18'!
timeAdvance

	^ Float infinity
! !

!MyContinuousBlock2 methodsFor: 'as yet unclassified' stamp: 'vj 9/1/2003 23:19'!
intgStep

	self intg: 1 integrate: [ intg at: 2 ].
 	self intg: 2 integrate: [ -9.81 * 1 ].

! !

!MyContinuousBlock2 methodsFor: 'as yet unclassified' stamp: 'vj 9/1/2003 18:32'!
outputFnc

	self poke: #e to: #o3.
! !

!MyContinuousBlock2 methodsFor: 'as yet unclassified' stamp: 'vj 9/1/2003 18:32'!
outputNFnc

	self poke: (intg at: 1) to: #o1.
	self poke: (intg at: 2) to: #o2.

! !

!MyContinuousBlock2 methodsFor: 'as yet unclassified' stamp: 'vj 9/6/2003 00:43'!
initConditions

	self allocateIntegrators: 2.
	intg at: 1 put: 1.0.
	intg at: 2 put: 0.0.

	self addSECondition: [ intg at: 1 ].

	self addOutputNPortNamed: #o1.
	self addOutputNPortNamed: #o2.
	self addOutputPortNamed: #o3.

! !

!MyContinuousBlock2 methodsFor: 'as yet unclassified' stamp: 'vj 9/6/2003 01:04'!
intTransition

	sePolarity < 0 ifFalse: [ ^self ].
	
	intg at: 1 put: 0.0.
	intg at: 2 put: -0.8 * (intg at: 2).
	self saveState. "is it needed??"
	! !

AtomicDEVnDESS subclass: #Plotter
	instanceVariableNames: 'pm pstep'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Continuous'!

!Plotter methodsFor: 'modeling' stamp: 'vj 9/20/2003 15:57'!
plotStep: t

	pstep _ t! !

!Plotter methodsFor: 'modeling' stamp: 'vj 9/20/2003 14:11'!
intTransition

	inputPorts keys do: [ :portName |
		pm serie: portName addPoint: (self rootSolver time) @ ((input at: (inputPorts at: portName))) ].
	pm title: self printString.	"may change at runtime"

! !

!Plotter methodsFor: 'modeling' stamp: 'vj 9/20/2003 15:56'!
timeAdvance

	^ pstep! !

!Plotter methodsFor: 'modeling' stamp: 'vj 8/27/2003 03:46'!
copy

	^ super copy open

! !

!Plotter methodsFor: 'modeling' stamp: 'vj 9/20/2003 13:55'!
outputNFnc! !

!Plotter methodsFor: 'modeling' stamp: 'vj 9/20/2003 19:36'!
initConditions

	pstep _ 0.1! !

!Plotter methodsFor: 'modeling' stamp: 'vj 8/27/2003 15:29'!
addPlotterPorts: listOfInputPortNames

	listOfInputPortNames do: [ :portName |
		inputPorts at: portName put: (InputNPort hostDEVS: self name: portName) ].
! !

!Plotter class methodsFor: 'as yet unclassified' stamp: 'vj 8/30/2003 19:59'!
withInputs: inputPortNames 

	| p |
	p _ super new.
	p addPlotterPorts: inputPortNames.
	^ p! !

!Plotter methodsFor: 'modeling' stamp: 'vj 10/28/2004 01:19'!
open

	pm _ PlotMorph new.
	pm position: (World extent x - 250) atRandom @ (World extent y - 250) atRandom.  
	pm extent: 250 @ 250.
	"pm color: (Color black alpha: 0.8)."
	inputPorts keys do: [ :portName | pm serie: portName color: Color random ].
	"pm title: self printString." 	"name will be known later"
	pm openInWorld

! !

!Plotter methodsFor: 'modeling' stamp: 'vj 9/20/2003 13:59'!
initialize

	super initialize.
	self open.
! !

!Plotter methodsFor: 'modeling' stamp: 'vj 9/20/2003 14:15'!
outputFnc! !

Plotter subclass: #XYPlotter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Continuous'!

!XYPlotter class methodsFor: 'as yet unclassified' stamp: 'vj 8/30/2003 19:45'!
withInputs: i! !

!XYPlotter class methodsFor: 'as yet unclassified' stamp: 'vj 8/30/2003 19:44'!
new 

	^ super withInputs: #(x y)! !

!XYPlotter methodsFor: 'as yet unclassified' stamp: 'vj 9/20/2003 14:21'!
intTransition

	pm serie: name addPoint: ((input at: (inputPorts at: #x))) @ ((input at: (inputPorts at: #y))).
	pm title: self printString.	"may change at runtime"
	^ output	
! !

!XYPlotter methodsFor: 'as yet unclassified' stamp: 'vj 10/28/2004 01:23'!
open

	pm _ PlotMorph new.
	pm position: (World extent x - 250) atRandom @ (World extent y - 250) atRandom.  
	pm extent: 250 @ 250.
	"pm color: (Color black alpha: 0.8)."
	"inputPorts keys do: [ :portName | pm serie: portName color: Color random ]."
	pm openInWorld

! !

!Plotter methodsFor: 'modeling' stamp: 'vj 9/20/2003 14:13'!
intgStep! !

AtomicDEVnDESS subclass: #MyContinuousBlock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Examples'!

!MyContinuousBlock commentStamp: 'vj 11/12/2003 23:18' prior: 0!
Atomic DEVnDESS example.!

!MyContinuousBlock methodsFor: 'as yet unclassified' stamp: 'vj 8/15/2003 01:26'!
outputFnc

	self poke: #e to: #o3.
! !

!MyContinuousBlock methodsFor: 'as yet unclassified' stamp: 'vj 8/5/2003 02:39'!
intgStep

	self intg: 1 integrate: [ intg at: 2 ].
 	self intg: 2 integrate: [ 1.0 * (1.0 - (intg at: 1) squared) * (intg at: 2) - (intg at: 1) ].

! !

!MyContinuousBlock methodsFor: 'as yet unclassified' stamp: 'vj 8/13/2003 19:02'!
intTransition

	! !

!MyContinuousBlock methodsFor: 'as yet unclassified' stamp: 'vj 8/13/2003 21:12'!
outputNFnc

	self poke: (intg at: 1) to: #o1.
	self poke: (intg at: 2) to: #o2.

! !

!MyContinuousBlock methodsFor: 'as yet unclassified' stamp: 'vj 8/15/2003 01:59'!
initConditions

	self allocateIntegrators: 2.

	intg at: 1 put: -2.0.
	intg at: 2 put: 2.0.

	self addSECondition: [ intg at: 1 ].

	self addOutputNPortNamed: #o1.
	self addOutputNPortNamed: #o2.

	self addOutputPortNamed: #o3.
	

! !

!MyContinuousBlock methodsFor: 'as yet unclassified' stamp: 'vj 12/5/2004 00:18'!
timeAdvance

	^ Float infinity
! !

!AtomicDEVnDESS methodsFor: 'as yet unclassified' stamp: 'vj 8/5/2003 21:37'!
revertToPrevState

	intg _ savedIntg copy.! !

!AtomicDEVnDESS methodsFor: 'as yet unclassified' stamp: 'vj 8/13/2003 01:29'!
allocateIntegrators: n

	intg _ Array new: n.
	intgTmp _ Array new: n.

! !

!AtomicDEVnDESS methodsFor: 'as yet unclassified' stamp: 'vj 9/20/2003 14:46'!
initialize

	super initialize.
	conditions _ OrderedCollection new.
	time _ step _ 0.
	"self allocateIntegrators: 0."

	self initConditions.		"subclass responsibility"

	self saveState.

! !

!AtomicDEVnDESS methodsFor: 'as yet unclassified' stamp: 'vj 9/6/2003 01:51'!
receiveNMsgX: x time: t step: h

	 | s1 s2 | 

	"integration step"
	input _ x.
	time _ t.
	step _ h.
	self intgStep.
	self intgPostStep.

	"check conditions"	
	seConditionIndex _ (1 to: conditions size) detect: [ :i |
		s1 _ (conditions at: i) value sign.
		s2 _ (savedConditionValues at: i) sign. 
		s1 ~= s2 ] ifNone: [ nil ].
	seConditionIndex isNil ifFalse: [ 
		sePolarity _ s1. 
		self sendMsgSETime: t + h ]

! !

!AtomicDEVnDESS methodsFor: 'as yet unclassified' stamp: 'vj 9/13/2003 22:00'!
receiveMsgInitTime: t

	super receiveMsgInitTime: t.
	time _ t.! !

!AtomicDEVnDESS methodsFor: 'as yet unclassified' stamp: 'vj 8/23/2003 16:54'!
intg: n integrate: block 

	"Euler integration. To be replaced by RK4 or something else in future."

	intgTmp at: n put: ((intg at: n) + (block value * step)).
! !

!AtomicDEVnDESS methodsFor: 'as yet unclassified' stamp: 'vj 8/26/2003 22:56'!
addOutputNPortNamed: n

	^ self addOutputPortOfType: OutputNPort named: n! !

!AtomicDEVS methodsFor: 'simulation' stamp: 'vj 9/20/2003 12:11'!
receiveMsgX: x time: t

	(timeLast <= t) & (t <= timeNext) ifFalse: [ self error: 'Bad Synchronization' ].

	input _ x.

	elapsed _ t - timeLast.
	self extTransition.

	timeLast _ t.
	timeNext _ timeLast + (savedTimeAdvance _ self timeAdvance).
	elapsed _ 0.

	verbose ifTrue: [
		reportStream nextPutAll: '* External Transition: ', self printString, String cr.
		reportStream nextPutAll: '    * Input Port Configuration: ', String cr.
		self reportInputPortConfig ].
! !

AtomicDEVS subclass: #AtomicDEVSPrototype
	instanceVariableNames: 'model'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Prototypes'!

!AtomicDEVSPrototype commentStamp: 'vj 11/3/2004 22:57' prior: 0!
Cloneable and editable version of AtomicDEVS for classless model construction.
Uses PrototypeObject from Prototype package.

!

!AtomicDEVSPrototype methodsFor: 'private' stamp: 'vj 11/3/2004 19:31'!
longPrintOn: aStream
	"Prints DEVS state. In this case, some slots of model."

	(model slotNames difference: #(atomic)) do: [ :title |
			aStream nextPutAll: '        ';
			 nextPutAll: title;
			 nextPut: $:;
			 space;
			 tab;
			 print: (model perform: title asSymbol);
			cr].




! !

!AtomicDEVSPrototype methodsFor: 'modeling' stamp: 'vj 11/3/2004 00:42'!
timeAdvance: code

	^ model addMethod: 'timeAdvance ', code.

! !

!AtomicDEVSPrototype class methodsFor: 'examples' stamp: 'vj 12/5/2004 00:14'!
example1
	"AtomicDEVSPrototype example1 simulate: 100"
	"An example of classless contruction of a DEVS model."
	| system generator processor jobPrototype |

	jobPrototype _ PrototypeObject new. 	
	jobPrototype addSlots: { 'n' -> 0.  'size' -> 0.  'name' -> 'aJob'. 'parent' -> nil }.
	jobPrototype addMethod: 'setSizeBetween: sl and: sh       
		self size: (sl to: sh) atRandom'.
	(SimSpace componentNames includes: 'Prototypes') ifFalse: [
		SimSpace addComponents: {	'Prototypes' -> SimSpace new. } ].
	(SimSpace componentNamed: 'Prototypes') addComponents: { 'jobPrototype'-> jobPrototype }.
	jobPrototype addSlots: { 'name' -> 'aJob' }.	 "little dirty hack: 'jobPrototype' renamed to 'aJob'"
	
	generator _ AtomicDEVSPrototype new.
	generator addSlots: {
		'ia' -> 2.			'ib' -> 7. 
		'sa' -> 5. 		'sb' 	-> 10.
		'first' ->	true.	'n' -> 0. }.	"n is number of jobs generated"
	generator intTransition: 'self first: false'.
	generator outputFnc:	'self n: self n +1. self poke: (	"(MyJob sizeBetween: self sa and: self sb)"
		(((SimSpace componentNamed: ''Prototypes'') componentNamed: ''jobPrototype'') 
		setSizeBetween: self sa and: self sb) clone 
							n: self n; yourself) to: #out'.		
	generator timeAdvance: '^ self first ifTrue: [ 0 ] ifFalse: [ (self ia to: self ib) atRandom ]'.
	generator addOutputPortNamed: #out.

	processor _ AtomicDEVSPrototype new.
	processor addSlots: { 
		'queue' -> OrderedCollection new.
		'queueSize' -> 5.
		'processorStatus' ->  #idle.
		'currentJob' ->  nil.
		'timeSpent' -> 0 }.
	processor addInputPortNamed: #in.
	processor addOutputPortNamed: #out.
	processor addOutputPortNamed: #discard.
	processor intTransition: '		
	  self processorStatus caseOf: {
		[ #busy ] -> [
			self queue size > 0 
				ifTrue: [ 
					self currentJob: (self queue removeFirst) ]
				ifFalse: [ 
					self processorStatus: #idle.
					self currentJob: nil ].
			self timeSpent: 0 ].
		[ #discard ] -> [
			self queue removeFirst.
			self queue size <= self queueSize ifTrue: [
				self processorStatus: #busy ]].
		[ #idle ] -> [ "nothing" ] } '.
	processor extTransition: '
	  self queue add: (self peekFrom: #in).		
  	  self processorStatus caseOf: {
		[ #idle ] -> [
			self processorStatus: #busy.
			self currentJob: (self queue removeFirst) ].
		[ #busy ] -> [
			self timeSpent: self timeSpent + self elapsed.
			self queue size > self queueSize ifTrue: [ self processorStatus: #discard ]].
		[ #discard ] -> [ "nothing" ] } '.
	processor outputFnc:	'
	  self processorStatus caseOf: {
		[ #busy ] -> [ self poke: self currentJob to: #out ].
		[ #discard ] -> [ self poke: (self queue last) to: #discard ].
		[ #idle ] -> [ "nothing" ] } '.		
	processor timeAdvance: '	
	  self processorStatus caseOf: {
		[ #busy ] 	-> [ ^ self currentJob size - self timeSpent ].
		[ #discard ] 	-> [ ^ 0 ].
		[ #idle ] 		-> [ ^ Float infinity ] }'.

	system _ CoupledDEVS new.
	system addOutputPorts: {
		#out.
		#discard }.
	system addComponents: {
		#generator -> generator.
		#processor -> processor }. 
	system addCouplings: { 
		#(generator out) 	-> #(processor in).
		#(processor out) 	-> #(self out).
		#(processor discard) -> #(self discard) }.

	^ (system getSimulator deepVerbose: true) 

! !

!AtomicDEVSPrototype methodsFor: 'private' stamp: 'vj 11/3/2004 00:35'!
intTransition

	^ model intTransition! !

!AtomicDEVSPrototype methodsFor: 'private' stamp: 'vj 11/3/2004 00:35'!
outputFnc

	^ model outputFnc

! !

!AtomicDEVSPrototype methodsFor: 'modeling' stamp: 'vj 11/3/2004 00:43'!
addSlot: slotName

	model addSlot: slotName! !

!AtomicDEVSPrototype methodsFor: 'modeling' stamp: 'vj 11/3/2004 00:41'!
extTransition: code

	^ model addMethod: 'extTransition ', code.

! !

!AtomicDEVSPrototype methodsFor: 'modeling' stamp: 'vj 11/3/2004 00:42'!
intTransition: code

	^ model addMethod: 'intTransition ', code.

! !

!AtomicDEVSPrototype methodsFor: 'modeling' stamp: 'vj 11/3/2004 00:52'!
initialize: code

	^ model addMethod: 'initialize ', code.

! !

!AtomicDEVSPrototype methodsFor: 'modeling' stamp: 'vj 11/3/2004 00:44'!
addMethod: code

	model addMethod: code! !

!AtomicDEVSPrototype methodsFor: 'modeling' stamp: 'vj 11/3/2004 00:55'!
addSlot: slotName withValue: value

	model addSlot: slotName withValue: value! !

!AtomicDEVSPrototype methodsFor: 'private' stamp: 'vj 11/3/2004 00:34'!
timeAdvance

	^ model timeAdvance
! !

!AtomicDEVSPrototype methodsFor: 'private' stamp: 'vj 11/3/2004 18:26'!
initialize
	super initialize.
	model ifNil: [ 
		model _ PrototypeObject new. 
		model addSlot: 'atomic' withValue: self.
		model addMethod: 'elapsed  ^ self atomic elapsed'.
		model addMethod: 'peekFrom: port   ^ self atomic peekFrom: port'.
		model addMethod: 'poke: value to: port   self atomic poke: value to: port' ].
	[ model initialize ] on: Error do: [].
! !

!AtomicDEVSPrototype methodsFor: 'modeling' stamp: 'vj 11/3/2004 21:44'!
addSlots: slots

	slots do: [ :assoc | model addSlot: assoc key withValue: assoc value ].
! !

!AtomicDEVSPrototype methodsFor: 'private' stamp: 'vj 11/3/2004 00:35'!
extTransition

	^ model extTransition
! !

!AtomicDEVSPrototype methodsFor: 'modeling' stamp: 'vj 11/3/2004 00:42'!
outputFnc: code

	^ model addMethod: 'outputFnc ', code.

! !

!AtomicDEVS methodsFor: 'modeling' stamp: 'vj 8/4/2003 18:38'!
intTransition

	self subclassResponsibility! !

!AtomicDEVS methodsFor: 'modeling' stamp: 'vj 7/17/2003 19:05'!
initialize

	super initialize.
	elapsed _ 0.

! !

AtomicDEVS subclass: #MyGenerator
	instanceVariableNames: 'ia ib sa sb first n'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Examples'!

!MyGenerator commentStamp: 'vj 7/21/2003 20:33' prior: 0!
Part of MySystem!

!MyGenerator methodsFor: 'as yet unclassified' stamp: 'vj 8/2/2003 00:46'!
timeAdvance

	^ first ifTrue: [ 0 ] ifFalse: [ (ia to: ib) atRandom ]! !

!MyGenerator methodsFor: 'as yet unclassified' stamp: 'vj 11/16/2004 16:47'!
initialize

	super initialize.

	first _ true.
	n _ 0.	"number of jobs generated"

	self addOutputPortNamed: #out
! !

!MyGenerator class methodsFor: 'as yet unclassified' stamp: 'vj 7/16/2003 18:25'!
ia: a ib: b sa: c sb: d

	^ self new ia: a ib: b sa: c sb: d
! !

MyGenerator subclass: #MyGenerator2
	instanceVariableNames: 'process'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Examples'!

!MyGenerator2 commentStamp: 'vj 11/12/2003 23:17' prior: 0!
Part of MySystem3b. Built fo RT.!

!MyGenerator2 methodsFor: 'as yet unclassified' stamp: 'vj 11/16/2004 16:01'!
initialize

	super initialize.

	process _ [ 
		[ true ] whileTrue: [
			(Delay forSeconds: 5) wait. 
			self rootSolver stateEventIn: self ].	"force root solver to send (*, t) to self"
	] newProcess.
	process priority: Processor userBackgroundPriority.	
	! !

!MyGenerator2 methodsFor: 'as yet unclassified' stamp: 'vj 11/16/2004 16:02'!
prepareToStop

	process suspend.
	! !

!MyGenerator2 methodsFor: 'as yet unclassified' stamp: 'vj 12/5/2004 00:19'!
timeAdvance

	^ Float infinity	! !

!MyGenerator2 methodsFor: 'as yet unclassified' stamp: 'vj 8/23/2003 23:07'!
finalize

	process isNil ifFalse: [ process terminate. process _ nil ]

! !

!MyGenerator2 methodsFor: 'as yet unclassified' stamp: 'vj 11/16/2004 16:21'!
prepareToStart

	process isNil ifTrue: [ self initialize ].
	process resume.
	! !

!MyGenerator methodsFor: 'as yet unclassified' stamp: 'vj 11/16/2004 16:47'!
ia: a ib: b sa: c sb: d

	ia _ a.	"interval min"
	ib _ b.	"interval max"
	sa _ c.	"job size min"
	sb _ d.	"job size max"

! !

!MyGenerator methodsFor: 'as yet unclassified' stamp: 'vj 7/16/2003 23:21'!
intTransition

	first _ false.
! !

!MyGenerator methodsFor: 'as yet unclassified' stamp: 'vj 7/22/2003 21:23'!
outputFnc

	n _ n+1.
	self poke: ((MyJob sizeBetween: sa and: sb) n: n) to: #out.! !

AtomicDEVS subclass: #MyProcessor
	instanceVariableNames: 'queue queueSize processorStatus currentJob timeSpent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Examples'!

!MyProcessor commentStamp: 'vj 7/21/2003 20:34' prior: 0!
Part of MySystem!

!MyProcessor methodsFor: 'as yet unclassified' stamp: 'vj 7/16/2003 19:11'!
queueSize: n

	queueSize _ n! !

!MyProcessor methodsFor: 'as yet unclassified' stamp: 'vj 8/14/2003 21:40'!
intTransition

	processorStatus caseOf: {
		[ #busy ] -> [
			queue size > 0 
				ifTrue: [ 
					currentJob _ queue removeFirst ]
				ifFalse: [ 
					processorStatus _ #idle.
					currentJob _ nil ].
			timeSpent _ 0 ].

		[ #discard ] -> [
			queue removeFirst.
			queue size <= queueSize ifTrue: [
				processorStatus _ #busy ]].

		[ #idle ] -> [ "nothing" ].
	}

! !

!MyProcessor methodsFor: 'as yet unclassified' stamp: 'vj 12/5/2004 00:19'!
timeAdvance

	processorStatus caseOf: {
		[ #busy ] 		-> [ ^ currentJob size - timeSpent ].
		[ #discard ] 	-> [ ^ 0 ].
		[ #idle ] 		-> [ ^ Float infinity ].
	} otherwise: [ self error ].
! !

!MyProcessor methodsFor: 'as yet unclassified' stamp: 'vj 8/14/2003 21:43'!
outputFnc

	processorStatus caseOf: {
		[ #busy ] -> [
			self poke: currentJob to: #out ].

		[ #discard ] -> [
			self poke: (queue last) to: #discard ].

		[ #idle ] -> [ "nothing" ].
	}

! !

!MyProcessor class methodsFor: 'as yet unclassified' stamp: 'vj 7/16/2003 20:49'!
withQueueSize: n

	^ super new initialize queueSize: n! !

!MyProcessor methodsFor: 'as yet unclassified' stamp: 'vj 8/14/2003 21:37'!
extTransition

	| job |

	job _ self peekFrom: #in.
	queue add: job.

	processorStatus caseOf: {
		[ #idle ] -> [
			processorStatus _ #busy.
			currentJob _ queue removeFirst ].

		[ #busy ] -> [
			timeSpent _ timeSpent + elapsed.
			queue size > queueSize ifTrue: [ processorStatus _ #discard ]].

		[ #discard ] -> [ "nothing" ]
	}

! !

!MyProcessor methodsFor: 'as yet unclassified' stamp: 'vj 7/17/2003 20:00'!
initialize

	super initialize.

	queue _ OrderedCollection new.
	queueSize _ 0.
	processorStatus _ #idle.
	currentJob _ nil.
	timeSpent _ 0.
	self addInputPortNamed: #in.
	self addOutputPortNamed: #out.
	self addOutputPortNamed: #discard.

! !

!AtomicDEVS methodsFor: 'simulation' stamp: 'vj 11/16/2004 15:58'!
receiveMsgSync

	self rootSolver isNil ifFalse: [ timeLast isNil ifFalse: [ 
		elapsed _ self rootSolver time - timeLast ]].

	self prepareToStop.		"suspend processes in atomic models"! !

AtomicDEVS subclass: #MyDaemon
	instanceVariableNames: 'done'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Examples'!

!MyDaemon commentStamp: 'vj 11/12/2003 23:12' prior: 0!
Part of coupled model MySystems. intTransition modifies structure of MySystems at runtime.
 !

!MyDaemon methodsFor: 'as yet unclassified' stamp: 'vj 11/21/2004 03:06'!
intTransition
	"This is an example of dynamic self-modication of a model."

	| newsystem newsolver |

	done ifTrue: [ self error: 'This shouldn''t occur'. ^self ].
	done _ true.

	"Add new components"
	newsystem _ MyContinuousSystem new.
	self rootSolver rootDEVS addComponents: { #ns -> newsystem }.

	"Add empty model and fill it by components ..."
	"note that all newly created models has to be initated with actual time"
	newsystem _ CoupledDEVnDESS new.	
	self rootSolver rootDEVS addComponents: { #ns2 -> newsystem }.
	newsystem addComponents: {
		#contiblock 	-> (MyContinuousBlock new).
		#plotter 	-> (Plotter withInputs: {#i1. #i2.}).
	}. 
	newsystem addCouplings: { 
		#(contiblock o1) -> #(plotter i1).
		#(contiblock o2) -> #(plotter i2).
	}.

	"Investigate actual model structure ..."
	"to be implemented: model structure accessing, state of atomic models accessing, ..."


	"Remove components"
	self rootSolver rootDEVS removeComponents: { #s1. #s2 }.

	"TO DO:"
	"It is possible to remove couplings, remove a model from some coupled model 
	 and add it to another coupled model"
	"It is also possible to compile new classes on the fly and use them here ..."
	"It would be interesting to create classes without names for that purpose ..."
	"... or rather self-contained objects a la Self ..."

	"Clone submodels ..."	
"	newsystem _ (self rootSolver rootDEVS subModelNamed: #c1) copy.
	self rootSolver rootDEVS addComponents: { #clone1 -> newsystem }.
"	newsystem _ (self rootSolver rootDEVS subModelNamed: #c2) copy.
	self rootSolver rootDEVS addComponents: { #clone2 -> newsystem }.

	"Clone the whole model ..."	
	newsystem _ self rootSolver rootDEVS copy.
	newsystem removeSubModelNamed: #d.
	self rootSolver rootDEVS addComponents: { #clone -> newsystem }.

	"Make nested simulation of a clone (with independent simulation time)" 		
	newsystem _ self rootSolver rootDEVS copy.
	newsolver _ self rootSolver copy rootDEVS: newsystem.
	newsystem name: #nestedSimulation.
	newsystem removeSubModelNamed: #d.
	newsolver simulate: 35. 			"starting at actual sim. time (25)"

	"Show actual model structure"
	reportStream nextPutAll: String cr, '* Model structure changed:'.
	self rootSolver rootDEVS reportStructure. 
	reportStream cr.

! !

!MyDaemon methodsFor: 'as yet unclassified' stamp: 'vj 8/26/2003 17:10'!
initialize

	super initialize.

	done _ false.

! !

!MyDaemon methodsFor: 'as yet unclassified' stamp: 'vj 12/5/2004 00:19'!
timeAdvance

	^ done ifTrue: [ Float infinity ] ifFalse: [ 25 ]! !

!MyDaemon methodsFor: 'as yet unclassified' stamp: 'vj 8/26/2003 17:04'!
outputFnc

! !

!AtomicDEVS methodsFor: 'modeling' stamp: 'vj 11/16/2004 16:00'!
prepareToStart
	"Resumes eventual processes in atomic model. It makes some sense in RT simulation."! !

!AtomicDEVS methodsFor: 'copying' stamp: 'vj 11/16/2004 13:54'!
copy

	| aCopy |
	self safelyDo: [ aCopy := super copy ].
	^ aCopy
! !

!AtomicDEVS methodsFor: 'modeling' stamp: 'vj 8/4/2003 18:38'!
timeAdvance

	self subclassResponsibility
! !

!AtomicDEVS methodsFor: 'modeling' stamp: 'vj 8/4/2003 18:38'!
outputFnc

	self subclassResponsibility
! !

!AtomicDEVS methodsFor: 'simulation' stamp: 'vj 9/20/2003 12:12'!
receiveMsgStarTime: t

	(t closeTo: timeNext) ifFalse: [ self error: 'Bad synchronization' ].

	output _ Dictionary new.
	self outputFnc.

	self intTransition.
	timeLast _ t.
	timeNext _ timeLast + (savedTimeAdvance _ self timeAdvance).
	elapsed _ 0.

	verbose ifTrue: [
		reportStream nextPutAll: '* Internal Transition: ', self printString, String cr.
		reportStream nextPutAll: '    * New State: ', String cr, self longPrintString.
		reportStream nextPutAll: '    * Output Port Configuration: ', String cr.
		self reportOutputPortConfig.
		reportStream nextPutAll: '    * Next scheduled internal transition at time ', timeNext printString, String cr ].

	^ output


! !

!AtomicDEVS methodsFor: 'printing' stamp: 'vj 8/4/2003 18:44'!
longPrintOn: aStream
	"Prints DEVS state, i.e. only variables that are not inherited from AtomicDEVS"

	self class allInstVarNames doWithIndex: [:title :index | 
		(AtomicDEVS allInstVarNames includes: title) ifFalse: [
			aStream nextPutAll: '        ';
			 nextPutAll: title;
			 nextPut: $:;
			 space;
			 tab;
			 print: (self instVarAt: index);
		 cr]]! !

!AtomicDEVS methodsFor: 'modeling' stamp: 'vj 11/16/2004 16:00'!
prepareToStop
	"Suspends eventual processes in atomic model. It makes some sense in RT simulation."
! !

!BaseDEVS methodsFor: 'misc' stamp: 'vj 8/12/2003 00:30'!
output
	^output! !

!BaseDEVS methodsFor: 'port accessing' stamp: 'vj 7/26/2003 18:36'!
peekFrom: p

	^ input at: (self inputPortNamed: p)! !

!BaseDEVS methodsFor: 'modeling' stamp: 'vj 8/27/2003 15:18'!
removeAllPorts

	inputPorts _ Dictionary new.
	outputPorts _ Dictionary new.
! !

!BaseDEVS methodsFor: 'printing' stamp: 'vj 8/29/2003 00:36'!
reportStructure

	| prefix |
	prefix _ (String cr), (String new: (self printString findTokens: '/') size withAll: Character tab).
	reportStream nextPutAll: prefix, (name isNil ifTrue: [''] ifFalse: [name]), ' (', self class printString, ')'.
	reportStream nextPutAll: ' ('.
	inputPorts do: [ :p | reportStream nextPutAll: ' ', p name ].
	reportStream nextPutAll: ' ) ('.
	outputPorts do: [ :p | reportStream nextPutAll: ' ', p name ].
	reportStream nextPutAll: ' )'.
	inputPorts do: [ :p | p outLines do: [ :l |
		reportStream nextPutAll: prefix, String tab, p name, ' -> ', l hostDEVS name, ' ', l name ]].
	outputPorts do: [ :p | p outLines do: [ :l |
		reportStream nextPutAll: prefix, String tab, p name, ' -> ', l hostDEVS name, ' ', l name ]].
! !

!BaseDEVS methodsFor: 'misc' stamp: 'vj 8/27/2003 02:58'!
rootSolver

	^ parent isNil ifTrue: [ nil ] ifFalse: [ parent rootSolver ]! !

!BaseDEVS methodsFor: 'simulation' stamp: 'vj 7/26/2003 17:43'!
reportStream

	^ reportStream
! !

!BaseDEVS methodsFor: 'private' stamp: 'vj 8/27/2003 15:28'!
addInputPortOfType: portClass named: n

	| port |
	port _ portClass hostDEVS: self name: n.
	inputPorts at: n put: port.
	^ port! !

!BaseDEVS methodsFor: 'misc' stamp: 'vj 9/20/2003 15:05'!
input: i
	input _ i! !

!BaseDEVS methodsFor: 'private' stamp: 'vj 11/19/2004 01:39'!
safelyDo: aBlock
	"Intended for model edition at runtime (not thread safe - usable either only reflectively by a DEVS itself during a simulation, or by anybody else when simulation is suspended; for thread safe use see AbstractSimulation>>safelyDo:). 
	Synchronizes DEVS before and inits it after the block evaluation. If the simulation is suspended, then no synchronization nor initialization is needed because it has already been synchronized during simulation stop and it will be initialized during simulation start."

	| v simulationIsRunning |

	self rootSolver notNil ifTrue: [ self rootSolver accessProtect critical: [
		simulationIsRunning := self rootSolver notNil and: [ self rootSolver isRunning ].	
		simulationIsRunning ifTrue: [ self receiveMsgSync ].
		v := aBlock value.
		simulationIsRunning ifTrue: [ self receiveMsgInitTime: self rootSolver time ] ] ]
	ifFalse: [ v := aBlock value ].
	^ v
! !

!BaseDEVS methodsFor: 'simulation' stamp: 'vj 9/20/2003 12:55'!
receiveMsgSync

	"See subclasses. MsgSync prepares atomic models to receive MsgInit safely during simulation. It updates elapsed time of atomic models. This is used before cloning, removing, and adding submodels during simulation. MsgInit uses saved value if timeAdvance for sure (it eases random generation of timeAdvance because timeAdvenace is not invocated more then once at a time).

(Things like this were not mentioned neither in [Uhrmacher:JAMES] nor [Kim&Kim] both dealing with dynamic modification of a model during simulation. JAMES initalizes only new components (model migration is not explicitly dealt with), Kim uses message Query, no Init (?), so elapsed time is not maintained (?). Theoretically it is possible to modify Kim's approach (MsgQuery) to deal with elapsed time instead of introducing MsgSync (?).)"


! !

!BaseDEVS methodsFor: 'simulation' stamp: 'vj 8/29/2003 03:06'!
simulate: t

	"obsolete!! Use getSimulator simulate: t "

	| clock |
	self receiveMsgInitTime: 0.
	clock _ timeNext.
	[ clock <= t ] whileTrue: [ 
		reportStream nextPutAll: '****************** time: ', clock printString, String cr.
		self receiveMsgStarTime: clock. 
		verbose ifTrue: [
			reportStream nextPutAll: '* Root DEVS Output Port Configuration: ', String cr.
		 	self reportOutputPortConfig ].
		clock _ self timeNext ].
	reportStream flush.


! !

!BaseDEVS methodsFor: 'modeling' stamp: 'vj 7/16/2003 20:44'!
outputPortNamed: n

	^ outputPorts at: n
! !

!BaseDEVS methodsFor: 'modeling' stamp: 'vj 8/25/2003 20:59'!
removeOutputPortNamed: n

	outputPorts removeKey: n.
! !

!BaseDEVS methodsFor: 'modeling' stamp: 'vj 7/16/2003 17:38'!
parent: p

	parent _ p
! !

!BaseDEVS class methodsFor: 'as yet unclassified' stamp: 'vj 3/18/2004 19:09'!
new

	^ self basicNew initialize	"not needed in 3.7+"! !

!BaseDEVS methodsFor: 'copying' stamp: 'vj 8/29/2003 18:59'!
storeOn: aStream

	"Caution: Does not work if BlockContexts are referenced by instance variables !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
	"Not tested."

	aStream nextPut: $(.
	aStream nextPutAll: self class name, ' basicNew'.
	(self class allInstVarNames difference: {'parent'. 'inputPorts'. 'outputPorts'. 'input'. 'output'.
		'reportStream'. 'components'. 'namedComponents'. 'eventList'}) do:
		[:n |
		aStream nextPutAll: ' instVarNamed: ';
			store: n;
			nextPutAll: ' put: ';
			store: (self instVarNamed: n);
			nextPut: $;].

	"ports"
	aStream nextPutAll: ' removeAllPorts;'.
	inputPorts keysAndValuesDo: [ :k :v | 
		aStream nextPutAll: ' addInputPortOfType: '; store: v species; nextPutAll: ' named: '; store: k; nextPut: $; ].	
	outputPorts keysAndValuesDo: [ :k :v | 
		aStream nextPutAll: ' addOutputPortOfType: '; store: v species; nextPutAll: ' named: '; store: k; nextPut: $; ].	

	aStream nextPutAll: ' yourself)'


! !

BaseDEVS subclass: #InstantaneousFunction
	instanceVariableNames: 'function inputPortNames'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Continuous'!

!InstantaneousFunction commentStamp: 'vj 8/13/2003 00:17' prior: 0!
Example - printIt:
| f |
f _ InstantaneousFunction withInputs: {#a. #b.} codeBlock: [:a :b | a+b ].
(f receiveNMsgX: (Dictionary newFrom: {#a->1.#b->2}) time: 0 step: 0) inspect.
f inspect.
!

!InstantaneousFunction methodsFor: 'simulation' stamp: 'vj 8/12/2003 03:32'!
receiveMsgStarTime: t

	^ Dictionary new.! !

!InstantaneousFunction methodsFor: 'modeling' stamp: 'vj 8/27/2003 15:29'!
addFunctionPorts: listOfInputPortNames


	inputPortNames _ listOfInputPortNames. 
	"we need to remember positions of parameter names"

	listOfInputPortNames do: [ :portName |
		inputPorts at: portName put: (InputNPort hostDEVS: self name: portName) ].

	outputPorts at: #output put: (OutputNPort hostDEVS: self name: #output).
	! !

!InstantaneousFunction methodsFor: 'modeling' stamp: 'vj 8/12/2003 01:59'!
function: aBlock

	function _ aBlock! !

!InstantaneousFunction methodsFor: 'simulation' stamp: 'vj 12/5/2004 00:16'!
receiveMsgInitTime: t

	timeLast _ t.
	timeNext _ Float infinity.! !

!InstantaneousFunction class methodsFor: 'as yet unclassified' stamp: 'vj 8/13/2003 00:07'!
withInputs: inputPortNames codeBlock: aBlock

	| function |
	function _ self new.
	function function: aBlock.
	function addFunctionPorts: inputPortNames.
	^ function! !

!InstantaneousFunction methodsFor: 'simulation' stamp: 'vj 8/13/2003 17:42'!
receiveNMsgX: x time: t step: h

	input _ x.

	self 
		poke: (function valueWithArguments: 
				(inputPortNames collect: [ :par | input at: (inputPorts at: par) ])) 
		to: #output.

	^ output	
! !

!DEVSEntity methodsFor: 'as yet unclassified' stamp: 'vj 7/21/2003 10:07'!
name

	^name! !

!DEVSEntity methodsFor: 'as yet unclassified' stamp: 'vj 7/22/2003 20:49'!
name: n

	name _ n. ! !

Object subclass: #SimSpace
	instanceVariableNames: 'parent name namedComponents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-SimSpace'!

SimSpace class
	instanceVariableNames: 'Root'!

!SimSpace commentStamp: 'vj 12/4/2004 18:14' prior: 0!
A folder of components. Contains a Dictionary of components. Each component must understand to protocol #name, #name:, #parent:, #parent.

SimSpace root (stored in a class variable) represents a root of a hierarchy of models and simulations. Models are e.g. DEVSes. Simulations are subclasses of AbstractSimulation. An exampe of a simulation is a CoupledDevs wrapped by DEVSRootSolver. The CoupledDEVS is a namespace - it contains named components. Each component in the hierarchy is globally identifiable by its pathname.

Example:

SimSpace Initialize.
SimSpace root addComponents: { 
	'Documents' -> SimSpace new. 
	'Prototypes' -> SimSpace new. 
	'Simulations' -> SimSpace new. 
	'TMP' -> SimSpace new. 
	'Trash' -> SimSpace new. }.
SimSpace root explore.
(SimSpace root componentNamed: 'Prototypes') explore.



To do: Protocol for editing ordering of components (or inherit Orderedollection????).

!

!SimSpace methodsFor: 'accessing' stamp: 'vj 11/6/2004 02:13'!
parent
	"Answer the value of parent"

	^ parent! !

!SimSpace class methodsFor: 'as yet unclassified' stamp: 'vj 12/4/2004 18:11'!
addComponents: compList

	^ self root addComponents: compList
! !

!SimSpace class methodsFor: 'as yet unclassified' stamp: 'vj 12/4/2004 18:12'!
componentNamed: n

	^ self root componentNamed: n! !

!SimSpace methodsFor: 'accessing' stamp: 'vj 11/14/2004 15:15'!
componentNamed: n

	n == #self ifTrue: [ ^ self ].
	^ namedComponents at: n! !

!SimSpace methodsFor: 'accessing' stamp: 'vj 11/21/2004 22:24'!
removeComponents: compList

	compList do: [ :x |
		namedComponents removeKey: x ].
! !

!SimSpace methodsFor: 'accessing' stamp: 'vj 11/6/2004 03:06'!
name: anObject

	name _ anObject! !

!SimSpace methodsFor: 'initializing' stamp: 'vj 11/21/2004 22:24'!
initialize

	namedComponents _ Dictionary new.! !

!SimSpace class methodsFor: 'as yet unclassified' stamp: 'vj 12/4/2004 18:12'!
componentNames

	^ self root componentNames! !

!SimSpace methodsFor: 'accessing' stamp: 'vj 11/14/2004 15:15'!
componentNames

	^ namedComponents keys! !

!SimSpace class methodsFor: 'as yet unclassified' stamp: 'vj 12/5/2004 02:33'!
Initialize
	Root isNil ifFalse: [ 
		(PopUpMenu confirm: 'Going to delete whole SimSpace hierrachy !!' orCancel: [^'Nevermind'])
			ifFalse: [ ^self ]]. 

	Root _ SimSpace new. Root name: 'Root'.

	Root addComponents: { 
		'Documents' -> SimSpace new. 
		'Traits' -> SimSpace new. 
		'Prototypes' -> SimSpace new. 
		'Simulations' -> SimSpace new. 
		'TMP' -> SimSpace new. 
		'Trash' -> SimSpace new. }.
! !

!SimSpace methodsFor: 'accessing' stamp: 'vj 11/6/2004 03:06'!
name

	^ name! !

!SimSpace methodsFor: 'accessing' stamp: 'vj 11/6/2004 03:05'!
parent: anObject

	parent _ anObject! !

!SimSpace methodsFor: 'accessing' stamp: 'vj 11/21/2004 22:24'!
addComponents: compList

	compList do: [ :assoc |
		namedComponents at: (assoc key) put: (assoc value).
		assoc value parent: self.
		assoc value name: assoc key ].
! !

!SimSpace class methodsFor: 'as yet unclassified' stamp: 'vj 12/4/2004 18:12'!
removeComponents: compList

	^ self root removeComponents: compList! !

!SimSpace class methodsFor: 'as yet unclassified' stamp: 'vj 12/5/2004 02:33'!
root
	Root isNil ifTrue: [ self Initialize ].
	^ Root
 
! !

Object subclass: #DEVSDocumentation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallDEVS-Examples'!

!DEVSDocumentation commentStamp: 'vj 1/25/2005 23:28' prior: 0!
Description:
	SmallDEVS - Multiparadigm simulation framework based on DEVS formalism. 
Author:
	Vladimir Janousek
Version:
	Just a snapshot of a work in progress. 

Comment:
	SmallDEVS is a new implementation of B. Zeigler's DEVS (Discrete systems specification) formalism and partially inspired by PythonDEVS (http://moncs.cs.mcgill.ca/MSDL/research/projects/DEVS/). 

What is DEVS good for? 
DEVS is a systems-theoretic approach to modeling. More specifically, it is state-centered formalism. System consists of interconnected subsystems. Leaf systems (atomic DEVSes) are state machines. Such a view on systems is mostly static - DEVS in its original form does not consider evolution of the structure. Only states of the state machines can evolve. Nevertheles, several later DEVS modificactions allow for structural changes. But in fact, DEVS seems to be more usable for mostly static structure systems modeling. Dynamic parts of the system is obviously better to encapsulate in atomic DEVSes where the subsystems dynamicity is modeled as state evolution. Inside an atomic DEVS, arbitray formalism or programming language can be used. This way, the DEVS can be viewed as a framework unifiyng a number of other formalisms in a consistent, systems theoretic, state centered fashion.

References:
	- Bernard P. Zeigler, Herbert Praehofer, Tag Gon Kim: "Theory of Modeling and Simulation. Integrating Discrete Event and Continuous Complex Dynamic Systems". Second Edition. Acadedemic Press, 2000
	- Python DEVS					http://moncs.cs.mcgill.ca/MSDL/research/projects/DEVS/
	- DEVS implementations 			http://www.sce.carleton.ca/faculty/wainer/standard/tools.htm

Design Notes:
	- Each (sub)model has its own simulator which is inherited from appropriate superclass. Models (together with their simulators) are hierarchically organized. Leafs of the tree are atomic models/simulators. Others are coupled models/coordinators. Top level model/simulator is controlled by a root-solver.
	- Numeric (continuous) simulation support is implemented by subclasses of discrete DEVS (in future, this could be better implemented using mixins). Note that the additional "N" in the middle of the appropriate message or class names means they are "numeric" counterparts the discrete ones. 
	- Numeric and discrete coordinators of DEV&DESS are glued together in one class - as oposed to Praehofer's STIMS-CLOS design, where numeric clusters are organized independently on discrete coordinators hierarchy.

Examples of DEVS models:
	DEVSDocumentation discreteDemo.		"You can watch the simulation in Transcript."
	DEVSDocumentation continuousDemo.	"Depends on PlotMorph from SqueakMap."
	DEVSDocumentation realtimeDemo.
	DEVSDocumentation dynamicDemo.
	DEVSDocumentation classlessDemo.		"Depends on Prototypes package from SqueakMap."
	DEVSDocumentation multisimulationDemo.

To do:
	- Documentation: How to model ...	 
	- Visualization: Oscilloscopes etc. 
	- Tools for interactive simulations: Potentiometer, Trigger, ... 
	- Adaptive integration method
	- Parallel & Distributed DEVS, ...
	- Interoperability with Scilab etc.

!

!DEVSDocumentation class methodsFor: 'demo' stamp: 'vj 12/4/2004 22:09'!
multisimulationDemo

	(SimSpace componentNamed: 'Simulations') addComponents: { 
		'AtomicDEVSPrototype example1' -> AtomicDEVSPrototype example1.
		'MySystem3b-RT' -> MySystem3b getSimulatorRT. 
		'MyContinuousSystem1' -> MyContinuousSystem1 getSimulator.
		'MyContinuousSystem2'-> MyContinuousSystem2 getSimulator. }.
	
	((SimSpace componentNamed: 'Simulations') componentNamed: 'MyContinuousSystem1') 
		stopTime: 5;
		start.

	((SimSpace componentNamed: 'Simulations') componentNamed: 'MyContinuousSystem2') 
		stopTime: 50;
		start.

	"Interactive simulation:
		SimSpace root explore.
	 Find a simulation and try:
		self start
		self stop
		self restart
		self rtFactor: 10
		self stopTime: 50
		self simulate: 20 "
! !

!DEVSDocumentation class methodsFor: 'demo' stamp: 'vj 9/20/2003 19:10'!
discreteDemo

	"Generator generates jobs of random size, cascade of processors with queues process the jobs."
	(MySystem getSimulator deepVerbose: true) simulate: 100.

	"dtto, one processor, another way of coupled model specification"
	(MySystem2 getSimulator deepVerbose: true) simulate: 100.

! !

!DEVSDocumentation class methodsFor: 'demo' stamp: 'vj 12/4/2004 18:36'!
classlessDemo

	"See AtomicDEVSPrototype>>example1 to learn how to construct DEVS models without classes."

	AtomicDEVSPrototype example1 simulate: 100

! !

!DEVSDocumentation class methodsFor: 'demo' stamp: 'vj 9/13/2003 20:40'!
continuousDemo

	"Van Der Pol system specified as atomic model, 
	 one of outputs trigger discrete events (when zero is crossed)"
	MyContinuousBlock getSimulator simulate: 20.

	"MyContinuousBlock with a plotter connected to it."
	MyContinuousSystem getSimulator simulate: 20.

	"Van Der Pol system specified as a coupling of integrators and a function"
	MyContinuousSystem2 getSimulator simulate: 20.

	"All systems in one"
	MySystems getSimulator simulate: 20.

	"Bouncing ball"
	MyContinuousSystem1 getSimulator simulate: 5

! !

!DEVSDocumentation class methodsFor: 'demo' stamp: 'vj 9/13/2003 18:23'!
realtimeDemo

	| s |
"	[ s _ MySystem3 getSimulatorRT. s simulate: 20. ] forkAt: Processor timingPriority.
	(Delay forSeconds: 10) wait.
	s inject: #aaaa to: #in.
"
	[ s _ MySystem3b getSimulatorRT. s simulate: 20. ] forkAt: Processor timingPriority.
! !

!DEVSDocumentation class methodsFor: 'demo' stamp: 'vj 9/20/2003 16:11'!
dynamicDemo

	| s c |
	s _ 	MySystems getSimulator simulate: 10.
	c _ s rootDEVS removeSubModelNamed: #c1.
	s simulate: 20.
	s rootDEVS addComponents: {#c1 -> c}.
	s simulate: 30.	"at t=25 model performs selfmodification - see MyDaemon>>intTransition"

! !

!PrototypeObject methodsFor: '*SmallDEVS' stamp: 'vj 11/3/2004 22:40'!
addSlots: anArrayOfAssociations

	anArrayOfAssociations do: [ :assoc |  self addSlot: assoc key withValue: assoc value ].
! !
